@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,11 +24,12 @@
 #include "gsaslserverbasic.h"
 #include "gsaslserverpam.h"
 
-std::unique_ptr<GAuth::SaslServer> GAuth::SaslServerFactory::newSaslServer( const SaslServerSecrets & secrets , const std::string & config , bool allow_apop )
+std::unique_ptr<GAuth::SaslServer> GAuth::SaslServerFactory::newSaslServer( const SaslServerSecrets & secrets ,
+	bool allow_pop , const std::string & config , bool no_insecure_mechanisms )
 {
 	if( secrets.source() == "/pam" )
-		return std::make_unique<SaslServerPam>(secrets,config,allow_apop) ; // up-cast
+		return std::make_unique<SaslServerPam>( secrets , allow_pop ) ;
 	else
-		return std::make_unique<SaslServerBasic>(secrets,config,allow_apop) ; // up-cast
+		return std::make_unique<SaslServerBasic>( secrets , allow_pop , config , no_insecure_mechanisms ) ;
 }
 
