@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,20 +21,44 @@
 #include "gdef.h"
 #include "gdaemon.h"
 #include "gprocess.h"
+#include "gfile.h"
 #include "gnewprocess.h"
+#include <chrono>
+#include <thread>
 
-void G::Daemon::detach( PidFile & pid_file )
+namespace G
 {
-	pid_file.check() ; // absolute path
-	detach() ;
+	namespace DaemonImp
+	{
+		void waitfor( const G::Path & pid_file )
+		{
+			if( !pid_file.empty() )
+			{
+				for( int i = 0 ; i < 100 ; i++ )
+				{
+					if( G::File::exists( pid_file , std::nothrow ) )
+						break ;
+					std::this_thread::sleep_for( std::chrono::milliseconds(10) ) ;
+				}
+			}
+		}
+	}
 }
 
 void G::Daemon::detach()
+{
+	detach( G::Path() ) ;
+}
+
+void G::Daemon::detach( const G::Path & pid_file )
 {
 	// see Stevens, ISBN 0-201-563137-7, ch 13.
 
 	if( !NewProcess::fork().first )
+	{
+		DaemonImp::waitfor( pid_file ) ; // because systemd
 		std::_Exit( 0 ) ; // exit from parent
+	}
 
 	setsid() ;
 	Process::cd( "/" , std::nothrow ) ;
