@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,13 +22,13 @@
 #include "gdaemon.h"
 #include "gprocess.h"
 
-void G::Daemon::detach( PidFile & )
-{
-	detach() ;
-}
-
 void G::Daemon::detach()
 {
 	// no-op
 }
 
+void G::Daemon::detach( const G::Path & )
+{
+	detach() ;
+}
+
