@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ void G::ExecutableCommand::osAddWrapper()
 		buffer.at(0) = '\0' ;
 		unsigned int n = ::GetWindowsDirectoryA( &buffer[0] , MAX_PATH ) ;
 		if( n == 0 || n > MAX_PATH )
-			throw std::runtime_error( "cannot determine the windows directory" ) ;
+			throw WindowsError() ;
 		windows = std::string( &buffer[0] , n ) ;
 	}
 
