@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ std::string G::hostname()
 	struct utsname info {} ;
 	int rc = ::uname( &info ) ;
 	if( rc == -1 )
-		return std::string() ;
+		return {} ;
 
 	std::string name = std::string( info.nodename ) ;
 	std::string::size_type pos = name.find( '.' ) ;
