@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,39 +22,40 @@
 #include "glog.h"
 #include "glogoutput.h"
 
-G::Log::Log( Severity severity , const char * file , int line ) :
+G::Log::Log( Severity severity , const char * file , int line ) noexcept :
 	m_severity(severity) ,
 	m_file(file) ,
 	m_line(line) ,
-	m_ostream(LogOutput::start(m_severity,m_file,m_line))
+	m_logstream(LogOutput::start(m_severity,m_file,m_line))
 {
+	static_assert( noexcept(LogStream(nullptr)) , "" ) ;
 }
 
 G::Log::~Log()
 {
-	try
-	{
-		LogOutput::output( m_ostream ) ;
-	}
-	catch(...)
-	{
-	}
+	static_assert( noexcept(LogOutput::output(m_logstream)) , "" ) ;
+	LogOutput::output( m_logstream ) ;
 }
 
-bool G::Log::at( Severity s )
+bool G::Log::at( Severity s ) noexcept
 {
+	static_assert( noexcept(LogOutput::instance()) , "" ) ;
 	const LogOutput * log_output = LogOutput::instance() ;
+
+	static_assert( noexcept(log_output->at(s)) , "" ) ;
 	return log_output && log_output->at( s ) ;
 }
 
-std::ostream & G::Log::operator<<( const char * s )
+G::LogStream & G::Log::operator<<( const char * s ) noexcept
 {
+	static_assert( noexcept(m_logstream <<s) , "" ) ;
 	s = s ? s : "" ;
-	return m_ostream << s ;
+	return m_logstream << s ;
 }
 
-std::ostream & G::Log::operator<<( const std::string & s )
+G::LogStream & G::Log::operator<<( const std::string & s ) noexcept
 {
-	return m_ostream << s ;
+	static_assert( noexcept(m_logstream <<s) , "" ) ;
+	return m_logstream << s ;
 }
 
