@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "gdef.h"
 #include "gmapfile.h"
 #include "gstr.h"
+#include "gstringtoken.h"
 #include "gpath.h"
 #include "gprocess.h"
 #include "gdatetime.h"
@@ -37,8 +38,8 @@
 G::MapFile::MapFile()
 = default;
 
-G::MapFile::MapFile( const G::Path & path , const std::string & kind ) :
-	m_kind(kind)
+G::MapFile::MapFile( const Path & path , string_view kind ) :
+	m_kind(sv_to_string(kind))
 {
 	if( !path.empty() )
 	{
@@ -52,7 +53,7 @@ G::MapFile::MapFile( std::istream & stream )
 	readFrom( stream ) ;
 }
 
-G::MapFile::MapFile( const G::StringMap & map ) :
+G::MapFile::MapFile( const StringMap & map ) :
 	m_map(map)
 {
 	m_keys.reserve( m_map.size() ) ;
@@ -60,14 +61,14 @@ G::MapFile::MapFile( const G::StringMap & map ) :
 		m_keys.push_back( p.first ) ;
 }
 
-G::MapFile::MapFile( const OptionMap & map , const std::string & yes )
+G::MapFile::MapFile( const OptionMap & map , string_view yes )
 {
 	for( auto p = map.begin() ; p != map.end() ; )
 	{
-		std::string key = (*p).first ;
+		const std::string & key = (*p).first ;
 		if( !(*p).second.isOff() )
 		{
-			std::string value = (*p).second.isOn() ? yes : map.value(key) ;
+			std::string value = (*p).second.isOn() ? sv_to_string(yes) : map.value(key) ;
 			add( key , value ) ;
 		}
 		while( p != map.end() && (*p).first == key ) // since we used OptionMap::value() to get them all
@@ -75,16 +76,16 @@ G::MapFile::MapFile( const OptionMap & map , const std::string & yes )
 	}
 }
 
-void G::MapFile::readFrom( const G::Path & path , const std::string & kind )
+void G::MapFile::readFrom( const Path & path , string_view kind )
 {
 	std::ifstream stream ;
 	File::open( stream , path , File::Text() ) ;
 	if( !stream.good() )
-		throw readError( path , kind ) ;
+		throw readError( path , sv_to_string(kind) ) ;
 	G_LOG( "MapFile::read: reading [" << path.str() << "]" ) ;
 	readFrom( stream ) ;
 	if( stream.bad() ) // eg. EISDIR
-		throw readError( path , kind ) ;
+		throw readError( path , sv_to_string(kind) ) ;
 }
 
 void G::MapFile::readFrom( std::istream & stream )
@@ -92,8 +93,8 @@ void G::MapFile::readFrom( std::istream & stream )
 	std::string line ;
 	while( stream.good() )
 	{
-		Str::readLineFrom( stream , "\n" , line ) ;
-		Str::trimRight( line , {"\r",1U} ) ;
+		Str::readLineFrom( stream , "\n"_sv , line ) ;
+		Str::trimRight( line , "\r"_sv ) ;
 		if( line.empty() )
 			continue ;
 		if( !stream )
@@ -102,22 +103,20 @@ void G::MapFile::readFrom( std::istream & stream )
 			continue ;
 
 		// no escaping here -- just strip quotes if the value starts and ends with them
-		std::string key ;
-		std::string value ;
-		{
-			StringArray parts ;
-			Str::splitIntoTokens( line , parts , " =\t" ) ;
-			if( parts.empty() )
-				continue ;
-			key = parts[0] ;
 
-			value = Str::tail( line , line.find(key)+key.size() , std::string() ) ;
-			Str::trimLeft( value , {" =\t",3U} ) ;
-			Str::trimRight( value , Str::ws() ) ;
+		G::string_view line_sv( line ) ;
+		StringTokenView t( line_sv , " =\t"_sv ) ;
+		if( !t.valid() )
+			continue ;
+
+		string_view key = t() ;
+		auto pos = line.find( key.data() , 0U , key.size() ) + key.size() ;
+		string_view value = Str::tailView( line , pos ) ;
+		value = Str::trimLeftView( value , " =\t"_sv ) ;
+		value = Str::trimRightView( value , Str::ws() ) ;
+		if( value.size() >= 2U && value.at(0U) == '"' && value.at(value.size()-1U) == '"' )
+			value = value.substr(1U,value.length()-2U) ;
 
-			if( value.length() >= 2U && value.at(0U) == '"' && value.at(value.length()-1U) == '"' )
-				value = value.substr(1U,value.length()-2U) ;
-		}
 		add( key , value ) ;
 	}
 }
@@ -129,13 +128,10 @@ bool G::MapFile::ignore( const std::string & line ) const
 		return true ;
 
 	std::string::size_type pos_hash = line.find('#') ;
-	if( pos_hash != std::string::npos && pos_hash < pos_interesting )
-		return true ;
-
-	return false ;
+	return pos_hash != std::string::npos && pos_hash < pos_interesting ;
 }
 
-void G::MapFile::check( const G::Path & path , const std::string & kind )
+void G::MapFile::check( const Path & path , string_view kind )
 {
 	MapFile tmp ;
 	tmp.readFrom( path , kind ) ;
@@ -146,7 +142,7 @@ void G::MapFile::log( const std::string & prefix_in ) const
 	std::string prefix = prefix_in.empty() ? std::string() : ( prefix_in + ": " ) ;
 	for( const auto & key : m_keys )
 	{
-		auto p = m_map.find( key ) ;
+		auto p = find( key ) ;
 		if( p == m_map.end() ) continue ;
 		std::string value = (*p).second ;
 		G_LOG( "MapFile::item: " << prefix << key << "=[" <<
@@ -157,13 +153,16 @@ void G::MapFile::log( const std::string & prefix_in ) const
 	}
 }
 
-void G::MapFile::writeItem( std::ostream & stream , const std::string & key ) const
+void G::MapFile::writeItem( std::ostream & stream , string_view key ) const
 {
-	std::string value = m_map.find(key) == m_map.end() ? std::string() : (*m_map.find(key)).second ;
-	writeItem( stream , key , value ) ;
+	auto p = find( key ) ;
+	if( p == m_map.end() )
+		writeItem( stream , key , {} ) ;
+	else
+		writeItem( stream , key , (*p).second ) ;
 }
 
-void G::MapFile::writeItem( std::ostream & stream , const std::string & key , const std::string & value )
+void G::MapFile::writeItem( std::ostream & stream , string_view key , string_view value )
 {
 	const char * qq = value.find(' ') == std::string::npos ? "" : "\"" ;
 	stream << key << "=" << qq << value << qq << "\n" ;
@@ -174,7 +173,7 @@ std::string G::MapFile::quote( const std::string & s )
 	return s.find_first_of(" \t") == std::string::npos ? s : ("\""+s+"\"") ;
 }
 
-void G::MapFile::editInto( const G::Path & path , bool make_backup ,
+void G::MapFile::editInto( const Path & path , bool make_backup ,
 	bool allow_read_error , bool allow_write_error ) const
 {
 	List lines = read( path , m_kind , allow_read_error ) ;
@@ -184,7 +183,7 @@ void G::MapFile::editInto( const G::Path & path , bool make_backup ,
 	save( path , lines , allow_write_error ) ;
 }
 
-G::MapFile::List G::MapFile::read( const G::Path & path , const std::string & kind , bool allow_read_error ) const
+G::MapFile::List G::MapFile::read( const Path & path , string_view kind , bool allow_read_error ) const
 {
 	List line_list ;
 	std::ifstream file_in ;
@@ -194,7 +193,7 @@ G::MapFile::List G::MapFile::read( const G::Path & path , const std::string & ki
 	while( file_in.good() )
 	{
 		std::string line = Str::readLineFrom( file_in , "\n" ) ;
-		Str::trimRight( line , {"\r",1U} ) ;
+		Str::trimRight( line , "\r"_sv ) ;
 		if( !file_in ) break ;
 		line_list.push_back( line ) ;
 	}
@@ -219,13 +218,13 @@ void G::MapFile::replace( List & line_list ) const
 		for( auto & line : line_list )
 		{
 			if( line.empty() ) continue ;
-			StringArray parts ;
-			Str::splitIntoTokens( line , parts , " \r\n\t=#" ) ;
-			if( parts.empty() ) continue ;
-			if( parts.at(0U) == map_item.first )
+			G::string_view line_sv( line ) ;
+			StringTokenView t( line_sv , " \r\n\t=#"_sv ) ;
+			if( !t ) continue ;
+			if( G::Str::match( map_item.first , t() ) )
 			{
-				std::string value = map_item.second ;
-				line = Str::trimmed( map_item.first + " " + quote(value) , Str::ws() ) ;
+				const std::string & value = map_item.second ;
+				line = Str::trimmed( std::string(map_item.first).append(1U,' ').append(quote(value)) , Str::ws() ) ;
 				found = true ;
 				break ;
 			}
@@ -233,13 +232,13 @@ void G::MapFile::replace( List & line_list ) const
 
 		if( !found )
 		{
-			std::string value = map_item.second ;
-			line_list.push_back( Str::trimmed( map_item.first + " " + quote(value) , Str::ws() ) ) ;
+			const std::string & value = map_item.second ;
+			line_list.push_back( Str::trimmed( std::string(map_item.first).append(1U,' ').append(quote(value)) , Str::ws() ) ) ;
 		}
 	}
 }
 
-void G::MapFile::backup( const G::Path & path )
+void G::MapFile::backup( const Path & path )
 {
 	// ignore errors
 	BrokenDownTime now = SystemTime::now().local() ;
@@ -249,7 +248,7 @@ void G::MapFile::backup( const G::Path & path )
 	File::copy( path , backup , std::nothrow ) ;
 }
 
-void G::MapFile::save( const G::Path & path , List & line_list , bool allow_write_error )
+void G::MapFile::save( const Path & path , List & line_list , bool allow_write_error )
 {
 	std::ofstream file_out ;
 	File::open( file_out , path , File::Text() ) ;
@@ -259,9 +258,9 @@ void G::MapFile::save( const G::Path & path , List & line_list , bool allow_writ
 		throw writeError( path ) ;
 }
 
-bool G::MapFile::booleanValue( const std::string & key , bool default_ ) const
+bool G::MapFile::booleanValue( string_view key , bool default_ ) const
 {
-	auto p = m_map.find( key ) ;
+	auto p = find( key ) ;
 	if( p == m_map.end() )
 	{
 		return default_ ;
@@ -272,58 +271,51 @@ bool G::MapFile::booleanValue( const std::string & key , bool default_ ) const
 	}
 	else
 	{
-		return G::Str::isPositive( (*p).second ) ;
+		return Str::isPositive( (*p).second ) ;
 	}
 }
 
-std::string G::MapFile::value( const std::string & key , const std::string & default_ ) const
+std::string G::MapFile::value( string_view key , string_view default_ ) const
 {
-	auto p = m_map.find( key ) ;
-	std::string result = ( p == m_map.end() || (*p).second.empty() ) ? default_ : (*p).second ;
-	return result ;
+	auto p = find( key ) ;
+	return ( p == m_map.end() || (*p).second.empty() ) ? sv_to_string(default_) : (*p).second ;
 }
 
-std::string G::MapFile::value( const std::string & key , const char * default_ ) const
+std::string G::MapFile::mandatoryValue( string_view key ) const
 {
-	return value( key , std::string(default_) ) ;
-}
-
-std::string G::MapFile::mandatoryValue( const std::string & key ) const
-{
-	if( m_map.find(key) == m_map.end() )
-		throw missingValueError( m_path , m_kind , key ) ;
+	if( find(key) == m_map.end() )
+		throw missingValueError( m_path , m_kind , sv_to_string(key) ) ;
 	return value( key ) ;
 }
 
-G::Path G::MapFile::pathValue( const std::string & key ) const
+G::Path G::MapFile::expandedPathValue( string_view key , const Path & default_ ) const
 {
-	return Path( mandatoryValue(key) ) ;
+	return { expand(value(key,default_.str())) } ;
 }
 
-G::Path G::MapFile::expandedPathValue( const std::string & key ) const
+G::Path G::MapFile::expandedPathValue( string_view key ) const
 {
-	return Path( expand(mandatoryValue(key)) ) ;
+	return { expand(mandatoryValue(key)) } ;
 }
 
-G::Path G::MapFile::pathValue( const std::string & key , const G::Path & default_ ) const
+G::Path G::MapFile::pathValue( string_view key , const Path & default_ ) const
 {
-	return Path( value(key,default_.str()) ) ;
+	return { value(key,default_.str()) } ;
 }
 
-G::Path G::MapFile::expandedPathValue( const std::string & key , const G::Path & default_ ) const
+G::Path G::MapFile::pathValue( string_view key ) const
 {
-	return Path( expand(value(key,default_.str())) ) ;
+	return { mandatoryValue(key) } ;
 }
 
-unsigned int G::MapFile::numericValue( const std::string & key , unsigned int default_ ) const
+unsigned int G::MapFile::numericValue( string_view key , unsigned int default_ ) const
 {
-	std::string s = value( key , std::string() ) ;
-	return !s.empty() && Str::isUInt(s) ? Str::toUInt( s ) : default_ ;
+	return Str::toUInt( value(key,{}) , default_ ) ;
 }
 
-void G::MapFile::remove( const std::string & key )
+void G::MapFile::remove( string_view key )
 {
-	auto p = m_map.find( key ) ;
+	auto p = find( key ) ;
 	if( p != m_map.end() )
 	{
 		m_map.erase( p ) ;
@@ -332,9 +324,9 @@ void G::MapFile::remove( const std::string & key )
 	}
 }
 
-std::string G::MapFile::expand( const std::string & value_in ) const
+std::string G::MapFile::expand( string_view value_in ) const
 {
-	std::string value( value_in ) ;
+	std::string value = sv_to_string(value_in) ;
 	expand_( value ) ;
 	return value ;
 }
@@ -388,7 +380,7 @@ bool G::MapFile::expand_( std::string & value ) const
 		if( end == npos ) break ;
 		end++ ;
 		std::string key = value.substr( start+1U , end-start-2U ) ;
-		auto p = m_map.find( key ) ;
+		auto p = find( key ) ;
 		if( p != m_map.end() )
 		{
 			std::size_t old = end - start ;
@@ -402,27 +394,38 @@ bool G::MapFile::expand_( std::string & value ) const
 	return changed ;
 }
 
-void G::MapFile::add( const std::string & key , const std::string & value , bool clear )
+void G::MapFile::add( string_view key_in , string_view value , bool clear )
 {
-	if( m_map.find(key) == m_map.end() )
+	std::string key = sv_to_string( key_in ) ;
+	if( find(key) == m_map.end() )
 	{
 		m_keys.push_back( key ) ;
-		m_map[key] = value ;
+		m_map[key] = sv_to_string(value) ;
 	}
 	else if( clear )
 	{
-		m_map[key] = value ;
+		m_map[key] = sv_to_string(value) ;
 	}
 	else
 	{
 		m_map[key].append( 1U , ',' ) ;
-		m_map[key].append( value ) ;
+		m_map[key].append( value.data() , value.size() ) ;
 	}
 }
 
-bool G::MapFile::contains( const std::string & key ) const
+G::StringMap::iterator G::MapFile::find( string_view key )
 {
-	return m_map.find( key ) != m_map.end() ;
+	return m_map.find( sv_to_string(key) ) ; // or c++14 'generic associative lookup' of string_view
+}
+
+G::StringMap::const_iterator G::MapFile::find( string_view key ) const
+{
+	return m_map.find( sv_to_string(key) ) ; // or c++14 'generic associative lookup' of string_view
+}
+
+bool G::MapFile::contains( string_view key ) const
+{
+	return find( key ) != m_map.end() ;
 }
 
 const G::StringMap & G::MapFile::map() const
@@ -435,32 +438,30 @@ const G::StringArray & G::MapFile::keys() const
 	return m_keys ;
 }
 
-std::string G::MapFile::ekind( const std::string & kind_in )
+std::string G::MapFile::ekind( string_view kind )
 {
-	return kind_in.empty() ? std::string("map") : kind_in ;
+	return kind.empty() ? std::string("map") : sv_to_string(kind) ;
 }
 
-std::string G::MapFile::epath( const G::Path & path_in )
+std::string G::MapFile::epath( const Path & path_in )
 {
 	return path_in.empty() ? std::string() : (" ["+path_in.str()+"]") ;
 }
 
-G::MapFile::Error G::MapFile::readError( const Path & path , const std::string & kind_in )
+G::MapFile::Error G::MapFile::readError( const Path & path , string_view kind )
 {
-	std::string description = "cannot read " + ekind(kind_in) + " file" + epath(path) ;
+	std::string description = "cannot read " + ekind(kind) + " file" + epath(path) ;
 	return Error( description ) ;
 }
 
-G::MapFile::Error G::MapFile::writeError( const Path & path , const std::string & kind_in )
+G::MapFile::Error G::MapFile::writeError( const Path & path , string_view kind )
 {
-	std::string description = "cannot create " + ekind(kind_in) + " file" + epath(path) ;
-	return Error( description ) ;
+	return Error( std::string("cannot create ").append(ekind(kind)).append(" file ").append(epath(path)) ) ;
 }
 
 G::MapFile::Error G::MapFile::missingValueError( const Path & path , const std::string & kind ,
 	const std::string & key )
 {
-	std::string description = "no item [" + key + "] in " + ekind(kind) + " file" + epath(path) ;
-	return Error( description ) ;
+	return Error( std::string("no item [").append(key).append("] in ").append(ekind(kind)).append(" file ").append(epath(path)) ) ;
 }
 
