@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,12 +23,12 @@
 
 #include "gdef.h"
 #include "gpath.h"
-#include "gstrings.h"
+#include "gstringarray.h"
+#include "gstringmap.h"
 #include "gexception.h"
 #include "goptions.h"
 #include "goptionvalue.h"
 #include "goptionmap.h"
-#include <map>
 #include <string>
 #include <iostream>
 
@@ -64,10 +64,10 @@ public:
 	MapFile() ;
 		///< Constructor for an empty map.
 
-	explicit MapFile( const G::StringMap & map ) ;
+	explicit MapFile( const StringMap & map ) ;
 		///< Constructor that initialises from a string map.
 
-	explicit MapFile( const OptionMap & map , const std::string & yes = std::string() ) ;
+	explicit MapFile( const OptionMap & map , string_view yes = {} ) ;
 		///< Constructor that initialises from an option value map,
 		///< typically parsed out from a command-line. Unvalued 'on'
 		///< options in the option value map are loaded into this
@@ -76,74 +76,73 @@ public:
 		///< Multi-valued options are loaded as a comma-separated
 		///< list.
 
-	explicit MapFile( const G::Path & , const std::string & kind = std::string() ) ;
+	explicit MapFile( const Path & , string_view kind = {} ) ;
 		///< Constructor that reads from a file. Lines can have a key
 		///< and no value (see booleanValue()). Comments must be at
 		///< the start of the line. Values are left and right-trimmed,
-		///< but can otherwise contain whitespace.
+		///< but can otherwise contain whitespace. The trailing parameter
+		///< is used in error messages to describe the kind of file,
+		///< defaulting to "map".
 
 	explicit MapFile( std::istream & ) ;
 		///< Constructor that reads from a stream.
 
-	const G::StringArray & keys() const ;
+	const StringArray & keys() const ;
 		///< Returns a reference to the internal ordered list of keys.
 
-	static void check( const G::Path & , const std::string & kind = std::string() ) ;
+	static void check( const Path & , string_view kind = {} ) ;
 		///< Throws if the file is invalid. This is equivalent to
 		///< constructing a temporary MapFile object, but it
 		///< specifically does not do any logging.
 
-	void add( const std::string & key , const std::string & value , bool clear = false ) ;
+	void add( string_view key , string_view value , bool clear = false ) ;
 		///< Adds or updates a single item in the map.
 		///< If updating then by default the new value
 		///< is appended with a comma separator.
 
-	void writeItem( std::ostream & , const std::string & key ) const ;
+	void writeItem( std::ostream & , string_view key ) const ;
 		///< Writes a single item from this map to the stream.
 
-	static void writeItem( std::ostream & , const std::string & key , const std::string & value ) ;
+	static void writeItem( std::ostream & , string_view key , string_view value ) ;
 		///< Writes an arbitrary item to the stream.
 
-	void editInto( const G::Path & path , bool make_backup ,
+	void editInto( const Path & path , bool make_backup ,
 		bool allow_read_error , bool allow_write_error ) const ;
 			///< Edits an existing file so that its contents
 			///< reflect this map.
 
-	bool contains( const std::string & key ) const ;
+	bool contains( string_view key ) const ;
 		///< Returns true if the map contains the given key.
 
-	G::Path pathValue( const std::string & key ) const ;
+	Path pathValue( string_view key ) const ;
 		///< Returns a mandatory path value from the map.
 		///< Throws if it does not exist.
 
-	G::Path pathValue( const std::string & key , const G::Path & default_ ) const ;
+	Path pathValue( string_view key , const Path & default_ ) const ;
 		///< Returns a path value from the map.
 
-	unsigned int numericValue( const std::string & key , unsigned int default_ ) const ;
+	unsigned int numericValue( string_view key , unsigned int default_ ) const ;
 		///< Returns a numeric value from the map.
 
-	std::string value( const std::string & key , const std::string & default_ = std::string() ) const ;
+	std::string value( string_view key , string_view default_ = {} ) const ;
 		///< Returns a string value from the map. Returns the default
 		///< if there is no such key or if the value is empty.
 
-	std::string value( const std::string & key , const char * default_ ) const ;
-		///< Returns a string value from the map.
-
-	bool booleanValue( const std::string & key , bool default_ ) const ;
+	bool booleanValue( string_view key , bool default_ ) const ;
 		///< Returns a boolean value from the map. Returns true if
 		///< the key exists with an empty value. Returns the default
 		///< if no such key.
 
-	void remove( const std::string & key ) ;
+	void remove( string_view key ) ;
 		///< Removes a value (if it exists).
 
-	const G::StringMap & map() const ;
+	const StringMap & map() const ;
 		///< Returns a reference to the internal map.
 
-	void log( const std::string & prefix = std::string() ) const ;
+	void log( const std::string & prefix = {} ) const ;
 		///< Logs the contents.
 
-	std::string expand( const std::string & value ) const ;
+	std::string expand( string_view value ) const ;
 		///< Does one-pass variable substitution for the given string.
 		///< Sub-strings like "%xyz%" are replaced by 'value("xyz")'
 		///< and "%%" is replaced by "%". If there is no appropriate
@@ -151,38 +150,40 @@ public:
 		///< (so "%xyz%" remains as "%xyz%" if there is no "xyz"
 		///< map item).
 
-	G::Path expandedPathValue( const std::string & key ) const ;
+	Path expandedPathValue( string_view key ) const ;
 		///< Returns a mandatory path value from the map
 		///< with expand(). Throws if it does not exist.
 
-	G::Path expandedPathValue( const std::string & key , const G::Path & default_ ) const ;
+	Path expandedPathValue( string_view key , const Path & default_ ) const ;
 		///< Returns a path value from the map with expand().
 
 private:
 	using List = std::list<std::string> ;
-	void readFrom( const G::Path & , const std::string & ) ;
+	void readFrom( const Path & , string_view ) ;
 	void readFrom( std::istream & ss ) ;
 	static std::string quote( const std::string & ) ;
-	List read( const G::Path & , const std::string & , bool ) const ;
+	List read( const Path & , string_view , bool ) const ;
 	void commentOut( List & ) const ;
 	void replace( List & ) const ;
 	bool expand_( std::string & ) const ;
-	std::string expandAll( const std::string & ) const ;
-	static void backup( const G::Path & ) ;
-	static void save( const G::Path & , List & , bool ) ;
-	std::string mandatoryValue( const std::string & ) const ;
+	std::string expandAll( string_view ) const ;
+	static void backup( const Path & ) ;
+	static void save( const Path & , List & , bool ) ;
+	std::string mandatoryValue( string_view ) const ;
 	bool ignore( const std::string & ) const ;
-	static std::string ekind( const std::string & ) ;
-	static std::string epath( const G::Path & ) ;
-	static Error readError( const G::Path & , const std::string & ) ;
-	static Error writeError( const G::Path & , const std::string & = std::string() ) ;
-	static Error missingValueError( const G::Path & , const std::string & , const std::string & ) ;
+	static std::string ekind( string_view ) ;
+	static std::string epath( const Path & ) ;
+	static Error readError( const Path & , string_view ) ;
+	static Error writeError( const Path & , string_view = {} ) ;
+	static Error missingValueError( const Path & , const std::string & , const std::string & ) ;
+	StringMap::const_iterator find( string_view ) const ;
+	StringMap::iterator find( string_view ) ;
 
 private:
-	G::Path m_path ; // if any
+	Path m_path ; // if any
 	std::string m_kind ;
-	G::StringMap m_map ;
-	G::StringArray m_keys ; // kept in input order
+	StringMap m_map ;
+	StringArray m_keys ; // kept in input order
 } ;
 
 #endif
