@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,10 +22,12 @@
 #define G_NET_EVENT_H
 
 #include "gdef.h"
-#include "gexceptionhandler.h"
 #include "geventloop.h"
 #include "geventhandler.h"
-#include "geventhandlerlist.h"
+#include "geventemitter.h"
+#include "gexceptionhandler.h"
+#include "gexceptionsink.h"
+#include "gexceptionsource.h"
 
 #endif
 
