@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,10 +28,9 @@
 #include <list>
 #include <algorithm>
 
-GNet::MultiServer::MultiServer( ExceptionSink es , const G::StringArray & interfaces , unsigned int port ,
-	const std::string & server_type , ServerPeerConfig server_peer_config , ServerConfig server_config ) :
+GNet::MultiServer::MultiServer( ExceptionSink es , const G::StringArray & listener_list , unsigned int port ,
+	const std::string & server_type , ServerPeer::Config server_peer_config , Server::Config server_config ) :
 		m_es(es) ,
-		m_interfaces(interfaces) ,
 		m_port(port) ,
 		m_server_type(server_type) ,
 		m_server_peer_config(server_peer_config) ,
@@ -39,43 +38,73 @@ GNet::MultiServer::MultiServer( ExceptionSink es , const G::StringArray & interf
 		m_if(es,*this) ,
 		m_interface_event_timer(*this,&MultiServer::onInterfaceEventTimeout,es)
 {
-	// tidy the interface list
-	std::sort( m_interfaces.begin() , m_interfaces.end() ) ;
-	m_interfaces.erase( std::unique(m_interfaces.begin(),m_interfaces.end()) , m_interfaces.end() ) ;
-
-	// build a listening address list from explicit addresses and/or interface names
-	G::StringArray used_names ; // interface names having one or more addresses
+	// parse the listener list
+	G::StringArray used_names ; // addresses, or interface names having one or more addresses
 	G::StringArray empty_names ; // interface names having no addresses
 	G::StringArray bad_names ; // non-address non-interface names
-	AddressList address_list = addresses( port , used_names , empty_names , bad_names ) ;
+	std::vector<int> fds ;
+	AddressList address_list ;
+	parse( listener_list , port , address_list , fds , used_names , empty_names , bad_names ) ;
 
 	// fail if any bad names
 	if( !bad_names.empty() )
 		throw InvalidName( bad_names.at(0) ) ;
 
-	// fail if no addresses and no prospect of getting any
-	if( address_list.empty() && ( empty_names.empty() || !Interfaces::active() ) )
-		throw NoListeningAddresses() ;
-
-	// warn if no addresses from one or more interface names
-	if( !empty_names.empty() )
+	// apply a default set of two wildcard addresses
+	if( fds.empty() && address_list.empty() && empty_names.empty() )
 	{
-		if( !address_list.empty() )
-			G_WARNING( "GNet::MultiServer::ctor: no addresses bound to named network interface"
-				<< (empty_names.size()==1U?"":"s")
-				<< " \"" << G::Str::join("\", \"",empty_names) << "\"" ) ;
-		else if( Interfaces::active() )
-			G_WARNING( "GNet::MultiServer::ctor: no listening addresses: waiting for interface"
-				<< (empty_names.size()>1U?"s ":" ")
-				<< "[" << G::Str::join("] [",empty_names) << "]" ) ;
+		if( Address::supports(Address::Family::ipv4) )
+			address_list.push_back( Address(Address::Family::ipv4,port) ) ;
+		if( Address::supports(Address::Family::ipv6) && StreamSocket::supports(Address::Family::ipv6) )
+			address_list.push_back( Address(Address::Family::ipv6,port) ) ;
+		if( address_list.empty() )
+			throw NoListeningAddresses() ;
 	}
 
-	// bind the listening addresses, etc
-	init( address_list ) ;
+	// fail if no addresses and no prospect of getting any
+	if( fds.empty() && address_list.empty() && !Interfaces::active() )
+		throw NoListeningAddresses() ;
+
+	// save used and empty names to query G::Interfaces later on
+	m_if_names.insert( m_if_names.end() , used_names.begin() , used_names.end() ) ;
+	m_if_names.insert( m_if_names.end() , empty_names.begin() , empty_names.end() ) ;
+	std::sort( m_if_names.begin() , m_if_names.end() ) ;
+	m_if_names.erase( std::unique(m_if_names.begin(),m_if_names.end()) , m_if_names.end() ) ;
+
+	// warn if no addresses from one or more interface names
+	if( !empty_names.empty() && !address_list.empty() )
+	{
+		G_WARNING( "GNet::MultiServer::ctor: no addresses bound to named network interface"
+			<< (empty_names.size()==1U?"":"s")
+			<< " \"" << G::Str::join("\", \"",empty_names) << "\"" ) ;
+	}
+
+	// warn if doing nothing until an interface come up
+	if( address_list.empty() && fds.empty() && !empty_names.empty() )
+	{
+		G_WARNING( "GNet::MultiServer::ctor: " << m_server_type << " server: nothing to do: "
+			<< "waiting for interface" << (empty_names.size()>1U?"s ":" ")
+			<< "[" << G::Str::join("] [",empty_names) << "]" ) ;
+	}
 
 	// warn if we got addresses from an interface name but won't get dynamic updates
 	if( !used_names.empty() && !Interfaces::active() )
-		G_WARNING_ONCE( "GNet::MultiServer::ctor: named network interfaces are not monitored for updates" ) ;
+	{
+		G_WARNING_ONCE( "GNet::MultiServer::ctor: named network interfaces "
+			"are not being monitored for address updates" ) ;
+	}
+
+	// instantiate the servers
+	for( const auto & fd : fds )
+	{
+		m_server_list.emplace_back( std::make_unique<MultiServerImp>( *this , m_es ,
+			Descriptor(fd) , m_server_peer_config , m_server_config ) ) ;
+	}
+	for( const auto & address : address_list )
+	{
+		m_server_list.emplace_back( std::make_unique<MultiServerImp>( *this , m_es ,
+			address , m_server_peer_config , m_server_config ) ) ;
+	}
 }
 
 GNet::MultiServer::~MultiServer()
@@ -83,6 +112,32 @@ GNet::MultiServer::~MultiServer()
 	serverCleanup() ;
 }
 
+void GNet::MultiServer::parse( const G::StringArray & listener_list , unsigned int port ,
+	AddressList & addresses_out , std::vector<int> & fds_out ,
+	G::StringArray & used_names_out , G::StringArray & empty_names_out , G::StringArray & bad_names_out )
+{
+	for( const auto & listener : listener_list )
+	{
+		if( listener.empty() ) continue ;
+		int fd = G::is_windows() ? -1 : parseFd( listener ) ;
+		if( fd >= 0 )
+			fds_out.push_back( fd ) ;
+		else
+			m_if.addresses( listener , port , addresses_out , &used_names_out , &empty_names_out , &bad_names_out ) ;
+	}
+}
+
+int GNet::MultiServer::parseFd( const std::string & listener )
+{
+	if( listener.size() > 3U && listener.find("fd#") == 0U && G::Str::isUInt(listener.substr(3U)) )
+	{
+		int fd = G::Str::toInt( listener.substr(3U) ) ;
+		if( fd < 0 ) throw InvalidFd( listener ) ;
+		return fd ;
+	}
+	return -1 ;
+}
+
 void GNet::MultiServer::serverCleanup()
 {
 	for( auto & server : m_server_list )
@@ -91,46 +146,10 @@ void GNet::MultiServer::serverCleanup()
 	}
 }
 
-std::vector<GNet::Address> GNet::MultiServer::addresses( unsigned int port ) const
-{
-	AddressList result ;
-	G::StringArray empty_names ;
-	G::StringArray used_names ;
-	G::StringArray bad_names ;
-	return addresses( port , used_names , empty_names , bad_names ) ;
-}
-
-std::vector<GNet::Address> GNet::MultiServer::addresses( unsigned int port , G::StringArray & used_names ,
-	G::StringArray & empty_names , G::StringArray & bad_names ) const
-{
-	AddressList result ;
-	if( m_interfaces.empty() )
-	{
-		if( Address::supports(Address::Family::ipv4) )
-			result.push_back( Address(Address::Family::ipv4,port) ) ;
-		if( Address::supports(Address::Family::ipv6) && StreamSocket::supports(Address::Family::ipv6) )
-			result.push_back( Address(Address::Family::ipv6,port) ) ;
-	}
-	else
-	{
-		result = m_if.addresses( m_interfaces , port , used_names , empty_names , bad_names ) ;
-	}
-	return result ;
-}
-
-void GNet::MultiServer::init( const AddressList & address_list )
-{
-	for( const auto & address : address_list )
-	{
-		m_server_list.emplace_back( std::make_unique<MultiServerImp>( *this , m_es , address , m_server_peer_config , m_server_config ) ) ;
-	}
-}
-
 void GNet::MultiServer::onInterfaceEvent( const std::string & /*description*/ )
 {
 	// notifications can be periodic and/or bursty, so minimal logging here
 	G_DEBUG( "GNet::MultiServer::onInterfaceEvent: network configuration change event" ) ;
-
 	m_if.load() ;
 	m_interface_event_timer.startTimer( 1U , 500000U ) ; // maybe increase for fewer bind warnings
 }
@@ -146,7 +165,8 @@ bool GNet::MultiServer::match( const Address & interface_address , const Address
 
 void GNet::MultiServer::onInterfaceEventTimeout()
 {
-	AddressList address_list = addresses( m_port ) ;
+	// get a fresh address list
+	AddressList address_list = m_if.addresses( m_if_names , m_port ) ;
 
 	// delete old
 	for( auto server_ptr_p = m_server_list.begin() ; server_ptr_p != m_server_list.end() ; )
@@ -187,7 +207,7 @@ void GNet::MultiServer::onInterfaceEventTimeout()
 				// (can fail here if notified too soon, but succeeds later)
 				G_LOG( "GNet::MultiServer::onInterfaceEvent: failed to bind " << displayString(address)
 					<< " for new " << m_server_type << " server:"
-					<< G::Str::tail(e.what(),std::string(e.what()).rfind(':')) ) ;
+					<< G::Str::tail(e.what(),G::string_view(e.what()).rfind(':')) ) ;
 			}
 		}
 	}
@@ -199,12 +219,6 @@ bool GNet::MultiServer::gotServerFor( const Address & interface_address ) const
 		[&interface_address](const ServerPtr &ptr){return match(interface_address,ptr->address());} ) ;
 }
 
-bool GNet::MultiServer::canBind( const AddressList & address_list , bool do_throw )
-{
-	return std::all_of( address_list.begin() , address_list.end() ,
-		[do_throw](const Address & a){return Server::canBind(a,do_throw);} ) ;
-}
-
 std::string GNet::MultiServer::displayString( const Address & address )
 {
 	return address.displayString( true ) ;
@@ -219,9 +233,9 @@ void GNet::MultiServer::serverReport() const
 }
 
 std::unique_ptr<GNet::ServerPeer> GNet::MultiServer::doNewPeer( ExceptionSinkUnbound esu ,
-	const ServerPeerInfo & pi , const ServerInfo & si )
+	ServerPeerInfo && pi , const ServerInfo & si )
 {
-	return newPeer( esu , pi , si ) ;
+	return newPeer( esu , std::move(pi) , si ) ;
 }
 
 bool GNet::MultiServer::hasPeers() const
@@ -234,7 +248,7 @@ bool GNet::MultiServer::hasPeers() const
 	return false ;
 }
 
-std::vector<std::weak_ptr<GNet::ServerPeer> > GNet::MultiServer::peers()
+std::vector<std::weak_ptr<GNet::ServerPeer>> GNet::MultiServer::peers()
 {
 	using List = std::vector<std::weak_ptr<ServerPeer>> ;
 	List result ;
@@ -249,10 +263,16 @@ std::vector<std::weak_ptr<GNet::ServerPeer> > GNet::MultiServer::peers()
 // ==
 
 GNet::MultiServerImp::MultiServerImp( MultiServer & ms , ExceptionSink es , const Address & address ,
-	ServerPeerConfig server_peer_config , ServerConfig server_config ) :
+	ServerPeer::Config server_peer_config , Server::Config server_config ) :
 		GNet::Server(es,address,server_peer_config,server_config) ,
-		m_ms(ms) ,
-		m_address(address)
+		m_ms(ms)
+{
+}
+
+GNet::MultiServerImp::MultiServerImp( MultiServer & ms , ExceptionSink es , Descriptor fd ,
+	ServerPeer::Config server_peer_config , Server::Config server_config ) :
+		GNet::Server(es,fd,server_peer_config,server_config) ,
+		m_ms(ms)
 {
 }
 
@@ -264,11 +284,11 @@ void GNet::MultiServerImp::cleanup()
 	serverCleanup() ;
 }
 
-std::unique_ptr<GNet::ServerPeer> GNet::MultiServerImp::newPeer( ExceptionSinkUnbound esu , ServerPeerInfo peer_info )
+std::unique_ptr<GNet::ServerPeer> GNet::MultiServerImp::newPeer( ExceptionSinkUnbound esu , ServerPeerInfo && peer_info )
 {
 	MultiServer::ServerInfo server_info ;
 	server_info.m_address = address() ; // GNet::Server::address()
-	return m_ms.doNewPeer( esu , peer_info , server_info ) ;
+	return m_ms.doNewPeer( esu , std::move(peer_info) , server_info ) ;
 }
 
 // ==
