@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -89,8 +89,8 @@ public:
 public:
 	ResolverFuture( const ResolverFuture & ) = delete ;
 	ResolverFuture( ResolverFuture && ) = delete ;
-	void operator=( const ResolverFuture & ) = delete ;
-	void operator=( ResolverFuture && ) = delete ;
+	ResolverFuture & operator=( const ResolverFuture & ) = delete ;
+	ResolverFuture & operator=( ResolverFuture && ) = delete ;
 
 private:
 	std::string failure() const ;
