@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,43 +22,19 @@
 #include "gsocket.h"
 #include "gtest.h"
 #include "gsleep.h"
+#include "glimits.h"
 #include "gmsg.h"
 #include "gstr.h"
 #include "gassert.h"
 #include "glog.h"
 
-namespace GNet
-{
-	namespace StreamSocketImp /// An implementation namespace for G::StreamSocket.
-	{
-		struct Options /// StreamSocket options.
-		{
-			enum class Linger { default_ , zero , nolinger } ;
-			Linger create_linger {Linger::nolinger} ;
-			bool create_keepalive { G::Test::enabled("socket-keepalive") } ;
-			Linger accept_linger {Linger::nolinger} ;
-			bool accept_keepalive { G::Test::enabled("socket-keepalive") } ;
-		} ;
-	}
-	namespace SocketImp /// An implementation namespace for G::Socket.
-	{
-		struct Options /// Socket options.
-		{
-			bool connect_pureipv6 {true} ;
-			bool bind_pureipv6 {true} ;
-			bool bind_reuse {true} ;
-			bool bind_exclusive { G::Test::enabled("socket-exclusive") } ;
-		} ;
-	}
-}
-
-// ==
-
 GNet::SocketBase::SocketBase( Address::Family family , int type , int protocol ) :
 	m_reason(0) ,
 	m_domain(Address::domain(family)) ,
 	m_family(family) ,
-	m_added(false) ,
+	m_read_added(false) ,
+	m_write_added(false) ,
+	m_other_added(false) ,
 	m_accepted(false)
 {
 	if( !create(m_domain,type,protocol) )
@@ -75,7 +51,9 @@ GNet::SocketBase::SocketBase( const SocketBase::Raw & , int domain , int type ,
 	m_reason(0) ,
 	m_domain(domain) ,
 	m_family(Address::Family::local) , // bogus value, see isFamily()
-	m_added(false) ,
+	m_read_added(false) ,
+	m_write_added(false) ,
+	m_other_added(false) ,
 	m_accepted(false)
 {
 	G_ASSERT( !Address::supports( Address::Domain() , domain ) ) ;
@@ -95,7 +73,9 @@ GNet::SocketBase::SocketBase( Address::Family family , Descriptor fd ) :
 	m_domain(Address::domain(family)) ,
 	m_family(family) ,
 	m_fd(fd) ,
-	m_added(false) ,
+	m_read_added(false) ,
+	m_write_added(false) ,
+	m_other_added(false) ,
 	m_accepted(false)
 {
 	if( !prepare(false) )
@@ -110,7 +90,9 @@ GNet::SocketBase::SocketBase( Address::Family family , Descriptor fd , const Acc
 	m_domain(Address::domain(family)) ,
 	m_family(family) ,
 	m_fd(fd) ,
-	m_added(false) ,
+	m_read_added(false) ,
+	m_write_added(false) ,
+	m_other_added(false) ,
 	m_accepted(true)
 {
 	if( !prepare(true) )
@@ -172,40 +154,46 @@ GNet::SocketBase::ssize_type GNet::SocketBase::writeImp( const char * buffer , s
 void GNet::SocketBase::addReadHandler( EventHandler & handler , ExceptionSink es )
 {
 	G_DEBUG( "GNet::SocketBase::addReadHandler: fd " << m_fd ) ;
-	EventLoop::instance().addRead( m_fd , handler , es ) ;
-	m_added = true ;
+	if( !m_read_added )
+		EventLoop::instance().addRead( m_fd , handler , es ) ;
+	m_read_added = true ;
 }
 
 void GNet::SocketBase::addWriteHandler( EventHandler & handler , ExceptionSink es )
 {
 	G_DEBUG( "GNet::SocketBase::addWriteHandler: fd " << m_fd ) ;
-	EventLoop::instance().addWrite( m_fd , handler , es ) ;
-	m_added = true ;
+	if( !m_write_added )
+		EventLoop::instance().addWrite( m_fd , handler , es ) ;
+	m_write_added = true ;
 }
 
 void GNet::SocketBase::addOtherHandler( EventHandler & handler , ExceptionSink es )
 {
 	G_DEBUG( "GNet::SocketBase::addOtherHandler: fd " << m_fd ) ;
-	EventLoop::instance().addOther( m_fd , handler , es ) ;
-	m_added = true ;
+	if( !m_other_added )
+		EventLoop::instance().addOther( m_fd , handler , es ) ;
+	m_other_added = true ;
 }
 
 void GNet::SocketBase::dropReadHandler() noexcept
 {
-	if( m_added && EventLoop::ptr() )
+	if( m_read_added && EventLoop::ptr() )
 		EventLoop::ptr()->dropRead( m_fd ) ;
+	m_read_added = false ;
 }
 
 void GNet::SocketBase::dropWriteHandler() noexcept
 {
-	if( m_added && EventLoop::ptr() )
+	if( m_write_added && EventLoop::ptr() )
 		EventLoop::ptr()->dropWrite( m_fd ) ;
+	m_write_added = false ;
 }
 
 void GNet::SocketBase::dropOtherHandler() noexcept
 {
-	if( m_added && EventLoop::ptr() )
+	if( m_other_added && EventLoop::ptr() )
 		EventLoop::ptr()->dropOther( m_fd ) ;
+	m_other_added = false ;
 }
 
 SOCKET GNet::SocketBase::fd() const noexcept
@@ -228,13 +216,24 @@ std::string GNet::SocketBase::asString() const
 
 // ==
 
-GNet::Socket::Socket( Address::Family af , int type , int protocol ) :
-	SocketBase(af,type,protocol)
+GNet::Socket::Config::Config()
+= default ;
+
+GNet::Socket::Socket( Address::Family af , int type , int protocol , const Config & config ) :
+	SocketBase(af,type,protocol) ,
+	m_config(config)
 {
 }
 
-GNet::Socket::Socket( Address::Family af , Descriptor s , const Accepted & a ) :
-	SocketBase(af,s,a)
+GNet::Socket::Socket( Address::Family af , Descriptor s , const Accepted & a , const Config & config ) :
+	SocketBase(af,s,a) ,
+	m_config(config)
+{
+}
+
+GNet::Socket::Socket( Address::Family af , Descriptor s , const Adopted & , const Config & config ) :
+	SocketBase(af,s) ,
+	m_config(config)
 {
 }
 
@@ -295,9 +294,6 @@ bool GNet::Socket::connect( const Address & address , bool * done )
 	{
 		saveReason() ;
 
-		if( G::Test::enabled("socket-slow-connect") )
-			sleep( 1 ) ;
-
 		if( eInProgress() )
 		{
 			G_DEBUG( "GNet::Socket::connect: connection in progress" ) ;
@@ -313,9 +309,13 @@ bool GNet::Socket::connect( const Address & address , bool * done )
 	return true ;
 }
 
-void GNet::Socket::listen( int backlog )
+void GNet::Socket::listen()
 {
-	int rc = ::listen( fd() , backlog ) ;
+	int listen_queue = m_config.listen_queue ;
+	if( listen_queue <= 0 )
+		listen_queue = G::limits::net_listen_queue ;
+
+	int rc = ::listen( fd() , std::max(1,listen_queue) ) ;
 	if( error(rc) )
 	{
 		saveReason() ;
@@ -323,6 +323,15 @@ void GNet::Socket::listen( int backlog )
 	}
 }
 
+GNet::Address GNet::Socket::getLocalAddress( Descriptor fd )
+{
+	AddressStorage address_storage ;
+	int rc = ::getsockname( fd.fd() , address_storage.p1() , address_storage.p2() ) ;
+	if( error(rc) )
+		throw SocketError( std::string("no bound address on fd ").append(std::to_string(fd.fd())) ) ;
+	return Address( address_storage ) ;
+}
+
 GNet::Address GNet::Socket::getLocalAddress() const
 {
 	AddressStorage address_storage ;
@@ -359,9 +368,7 @@ void GNet::Socket::setOptionsOnConnect( Address::Family af )
 {
 	if( af == Address::Family::ipv4 || af == Address::Family::ipv6 )
 	{
-		using namespace SocketImp ;
-		Options options ;
-		if( af == Address::Family::ipv6 && options.connect_pureipv6 )
+		if( af == Address::Family::ipv6 && m_config.connect_pureipv6 )
 			setOptionPureV6( std::nothrow ) ; // ignore errors - may fail if already bound
 	}
 }
@@ -370,13 +377,13 @@ void GNet::Socket::setOptionsOnBind( Address::Family af )
 {
 	if( af == Address::Family::ipv4 || af == Address::Family::ipv6 )
 	{
-		using namespace SocketImp ;
-		Options options ;
-		if( options.bind_reuse )
-			setOptionReuse() ; // allow us to rebind another socket's (eg. time-wait zombie's) address
-		if( options.bind_exclusive )
-			setOptionExclusive() ; // don't allow anyone else to bind our address
-		if( af == Address::Family::ipv6 && options.bind_pureipv6 )
+		if( m_config.bind_reuse )
+			setOptionReuse() ;
+		if( m_config.bind_exclusive )
+			setOptionExclusive() ;
+		if( m_config.free_bind )
+			setOptionFreeBind() ;
+		if( af == Address::Family::ipv6 && m_config.bind_pureipv6 )
 			setOptionPureV6() ;
 	}
 }
@@ -386,21 +393,22 @@ void GNet::Socket::setOptionKeepAlive()
 	setOption( SOL_SOCKET , "so_keepalive" , SO_KEEPALIVE , 1 ) ;
 }
 
-void GNet::Socket::setOptionNoLinger()
+void GNet::Socket::setOptionFreeBind()
 {
-	setOptionLingerImp( 0 , 0 ) ;
+	// not tested -- can also use /proc
+	//setOption( IPPROTO_IP , "so_freebind" , IP_FREEBIND , 1 ) ;
 }
 
-void GNet::Socket::setOptionLingerImp( int onoff , int time )
+void GNet::Socket::setOptionLinger( int onoff , int time )
 {
-	struct linger options {} ;
-	options.l_onoff = onoff ;
-	options.l_linger = time ;
-	bool ok = setOptionImp( SOL_SOCKET , SO_LINGER , &options , sizeof(options) ) ;
+	struct linger linger_config {} ;
+	linger_config.l_onoff = onoff ;
+	linger_config.l_linger = time ;
+	bool ok = setOptionImp( SOL_SOCKET , SO_LINGER , &linger_config , sizeof(linger_config) ) ;
 	if( !ok )
 	{
 		saveReason() ;
-		throw SocketError( "cannot set no_linger" , reason() ) ;
+		throw SocketError( "cannot set socket linger option" , reason() ) ;
 	}
 }
 
@@ -421,6 +429,14 @@ void GNet::Socket::setOption( int level , const char * opp , int op , int arg )
 
 //==
 
+GNet::StreamSocket::Config::Config()
+= default ;
+
+GNet::StreamSocket::Config::Config( const Socket::Config & base ) :
+	Socket::Config(base)
+{
+}
+
 bool GNet::StreamSocket::supports( Address::Family af )
 {
 	if( af == Address::Family::ipv6 )
@@ -449,24 +465,38 @@ bool GNet::StreamSocket::supports( Address::Family af )
 	}
 }
 
-GNet::StreamSocket::StreamSocket( Address::Family af ) :
-	Socket(af,SOCK_STREAM,0)
+GNet::StreamSocket::StreamSocket( Address::Family af , const Config & config ) :
+	Socket(af,SOCK_STREAM,0,config) ,
+	m_config(config)
 {
 	setOptionsOnCreate( af , /*listener=*/false ) ;
 }
 
-GNet::StreamSocket::StreamSocket( Address::Family af , const Listener & ) :
-	Socket(af,SOCK_STREAM,0)
+GNet::StreamSocket::StreamSocket( Address::Family af , const Listener & , const Config & config ) :
+	Socket(af,SOCK_STREAM,0,config) ,
+	m_config(config)
 {
 	setOptionsOnCreate( af , /*listener=*/true ) ;
 }
 
-GNet::StreamSocket::StreamSocket( Address::Family af , Descriptor s , const Accepted & accepted ) :
-	Socket(af,s,accepted)
+GNet::StreamSocket::StreamSocket( const Listener & , Descriptor fd , const Config & config ) :
+	Socket(family(fd),fd,Socket::Adopted(),config) ,
+	m_config(config)
+{
+}
+
+GNet::StreamSocket::StreamSocket( Address::Family af , Descriptor fd , const Accepted & accepted , const Config & config ) :
+	Socket(af,fd,accepted,config) ,
+	m_config(config)
 {
 	setOptionsOnAccept( af ) ;
 }
 
+GNet::Address::Family GNet::StreamSocket::family( Descriptor fd )
+{
+	return getLocalAddress(fd).family() ;
+}
+
 GNet::Socket::ssize_type GNet::StreamSocket::read( char * buffer , size_type length )
 {
 	if( length == 0 ) return 0 ;
@@ -486,7 +516,7 @@ GNet::Socket::ssize_type GNet::StreamSocket::write( const char * buffer , size_t
 	return writeImp( buffer , length ) ; // SocketBase
 }
 
-GNet::AcceptPair GNet::StreamSocket::accept()
+GNet::AcceptInfo GNet::StreamSocket::accept()
 {
 	AddressStorage addr ;
 	Descriptor new_fd( ::accept(fd(),addr.p1(),addr.p2()) ) ;
@@ -502,9 +532,9 @@ GNet::AcceptPair GNet::StreamSocket::accept()
 	if( G::Test::enabled("socket-accept-throws") )
 		throw SocketError( "testing" ) ;
 
-	AcceptPair info ;
+	AcceptInfo info ;
 	info.address = Address( addr ) ;
-	info.socket_ptr.reset( new StreamSocket( info.address.family() , new_fd , SocketBase::Accepted() ) ) ; // 'new' for access
+	info.socket_ptr.reset( new StreamSocket( info.address.family() , new_fd , SocketBase::Accepted() , m_config ) ) ; // 'new' sic
 
 	G_DEBUG( "GNet::StreamSocket::accept: accepted from " << fd()
 		<< " to " << new_fd << " (" << info.address.displayString() << ")" ) ;
@@ -516,13 +546,11 @@ void GNet::StreamSocket::setOptionsOnCreate( Address::Family af , bool /*listene
 {
 	if( af == Address::Family::ipv4 || af == Address::Family::ipv6 )
 	{
-		using namespace StreamSocketImp ;
-		Options options ;
-		if( options.create_linger == Options::Linger::zero )
-			setOptionLingerImp( 1 , 0 ) ;
-		else if( options.create_linger == Options::Linger::nolinger )
-			setOptionNoLinger() ;
-		if( options.create_keepalive )
+		if( m_config.create_linger_onoff == 1 )
+			setOptionLinger( 1 , m_config.create_linger_time ) ;
+		else if( m_config.create_linger_onoff == 0 )
+			setOptionLinger( 0 , 0 ) ;
+		if( m_config.create_keepalive )
 			setOptionKeepAlive() ;
 	}
 }
@@ -531,21 +559,27 @@ void GNet::StreamSocket::setOptionsOnAccept( Address::Family af )
 {
 	if( af == Address::Family::ipv4 || af == Address::Family::ipv6 )
 	{
-		using namespace StreamSocketImp ;
-		Options options ;
-		if( options.accept_linger == Options::Linger::zero )
-			setOptionLingerImp( 1 , 0 ) ;
-		else if( options.accept_linger == Options::Linger::nolinger )
-			setOptionNoLinger() ;
-		if( options.accept_keepalive )
+		if( m_config.accept_linger_onoff == 1 )
+			setOptionLinger( 1 , m_config.accept_linger_time ) ;
+		else if( m_config.accept_linger_onoff == 0 )
+			setOptionLinger( 0 , 0 ) ;
+		if( m_config.accept_keepalive )
 			setOptionKeepAlive() ;
 	}
 }
 
 //==
 
-GNet::DatagramSocket::DatagramSocket( Address::Family af , int protocol ) :
-	Socket( af , SOCK_DGRAM , protocol )
+GNet::DatagramSocket::Config::Config()
+= default ;
+
+GNet::DatagramSocket::Config::Config( const Socket::Config & base ) :
+	Socket::Config(base)
+{
+}
+
+GNet::DatagramSocket::DatagramSocket( Address::Family af , int protocol , const Config & config ) :
+	Socket(af,SOCK_DGRAM,protocol,config)
 {
 }
 
