@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -132,14 +132,65 @@ bool GNet::SocketBase::setNonBlocking()
 
 std::string GNet::SocketBase::reasonString( int e )
 {
-	return G::Process::strerror( e ) ;
+	const char * p = nullptr ;
+	if( e == WSAEINTR ) p = "interupted" ;
+	//if( e == WSAEBADF ) p = "" ;
+	if( e == WSAEACCES ) p = "access denied" ;
+	//if( e == WSAEFAULT ) p = "" ;
+	if( e == WSAEINVAL ) p = "invalid parameter" ;
+	//if( e == WSAEMFILE ) p = "" ;
+	//if( e == WSAEWOULDBLOCK ) p = "" ;
+	//if( e == WSAEINPROGRESS ) p = "" ;
+	//if( e == WSAEALREADY ) p = "" ;
+	//if( e == WSAENOTSOCK ) p = "" ;
+	//if( e == WSAEDESTADDRREQ ) p = "" ;
+	//if( e == WSAEMSGSIZE ) p = "" ;
+	//if( e == WSAEPROTOTYPE ) p = "" ;
+	//if( e == WSAENOPROTOOPT ) p = "" ;
+	//if( e == WSAEPROTONOSUPPORT ) p = "" ;
+	//if( e == WSAESOCKTNOSUPPORT ) p = "" ;
+	//if( e == WSAEOPNOTSUPP ) p = "" ;
+	//if( e == WSAEPFNOSUPPORT ) p = "" ;
+	//if( e == WSAEAFNOSUPPORT ) p = "" ;
+	if( e == WSAEADDRINUSE ) p = "address already in use" ;
+	if( e == WSAEADDRNOTAVAIL ) p = "address not available" ;
+	if( e == WSAENETDOWN ) p = "network down" ;
+	if( e == WSAENETUNREACH ) p = "network unreachable" ;
+	if( e == WSAENETRESET ) p = "network reset" ;
+	//if( e == WSAECONNABORTED ) p = "" ;
+	//if( e == WSAECONNRESET ) p = "" ;
+	//if( e == WSAENOBUFS ) p = "" ;
+	//if( e == WSAEISCONN ) p = "" ;
+	if( e == WSAENOTCONN ) p = "cannot connect" ;
+	//if( e == WSAESHUTDOWN ) p = "" ;
+	//if( e == WSAETOOMANYREFS ) p = "" ;
+	if( e == WSAETIMEDOUT ) p = "timed out" ;
+	if( e == WSAECONNREFUSED ) p = "connection refused" ;
+	//if( e == WSAELOOP ) p = "" ;
+	//if( e == WSAENAMETOOLONG ) p = "" ;
+	if( e == WSAEHOSTDOWN ) p = "host down" ;
+	if( e == WSAEHOSTUNREACH ) p = "host unreachable" ;
+	//if( e == WSAENOTEMPTY ) p = "" ;
+	//if( e == WSAEPROCLIM ) p = "" ;
+	//if( e == WSAEUSERS ) p = "" ;
+	//if( e == WSAEDQUOT ) p = "" ;
+	//if( e == WSAESTALE ) p = "" ;
+	//if( e == WSAEREMOTE ) p = "" ;
+	if( p )
+		return std::string( p ) ;
+
+	std::string s = G::Process::strerror( e ) ;
+	if( G::Str::imatch(s,"unknown error") )
+		return s.append(" (").append(G::Str::fromInt(e)).append(1U,')') ;
+	else
+		return s ;
 }
 
 // ==
 
-std::string GNet::Socket::canBindHint( const Address & , bool )
+std::string GNet::Socket::canBindHint( const Address & , bool , const Config & )
 {
-	return std::string() ; // rebinding the same port number fails, so a dummy implementation here
+	return std::string() ; // not implemented
 }
 
 void GNet::Socket::setOptionReuse()
@@ -166,6 +217,7 @@ bool GNet::Socket::setOptionImp( int level , int op , const void * arg , socklen
 {
 	const char * cp = static_cast<const char*>(arg) ;
 	int rc = ::setsockopt( fd() , level , op , cp , n ) ;
-	return ! error(rc) ;
+	bool ok = !error( rc ) ;
+	return ok ;
 }
 
