@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "gdef.h"
 #include "gverifier.h"
 #include "gexceptionsink.h"
+#include "gfactoryparser.h"
 #include <string>
 #include <utility>
 #include <memory>
@@ -40,14 +41,9 @@ class GSmtp::VerifierFactory
 {
 public:
 	static std::unique_ptr<Verifier> newVerifier( GNet::ExceptionSink ,
-		const std::string & identifier , unsigned int timeout ) ;
-			///< Returns a Verifier on the heap. The identifier
-			///< is normally prefixed with a verifier type, or it
-			///< is the file system path of an exectuable.
-
-	static std::string check( const std::string & identifier ) ;
-		///< Checks an identifier. Returns an empty string if okay,
-		///< or a diagnostic reason string.
+		const FactoryParser::Result & spec , unsigned int timeout ) ;
+			///< Returns a Verifier on the heap. Throws if
+			///< an invalid or unsupported specification.
 
 public:
 	VerifierFactory() = delete ;
