@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "gdef.h"
 #include "gcominit.h"
 #include "gstr.h"
+#include "gstringmap.h"
 #include "gpath.h"
 #include "gfile.h"
 #include "gdate.h"
@@ -28,16 +29,18 @@
 #include "gtime.h"
 #include "gdirectory.h"
 #include "gprocess.h"
-#include "glink.h"
+#include "guilink.h"
 #include "gbatchfile.h"
+#include "gxtext.h"
+#include "gbase64.h"
 #include "glog.h"
 #include "glogoutput.h"
 #include "gexception.h"
 #include "gexecutablecommand.h"
 #include "gnewprocess.h"
 #include "gmapfile.h"
-#include "boot.h"
-#include "access.h"
+#include "guiboot.h"
+#include "guiaccess.h"
 #include "serverconfiguration.h"
 #include "installer.h"
 #include <functional>
@@ -56,7 +59,7 @@
 using trstring = QString ;
 static std::string from_trstring( const trstring & qs )
 {
-	return GQt::stdstr( qs , GQt::Utf8() ) ;
+	return GQt::stdstr( qs , GQt::Utf8 ) ;
 }
 static bool empty( const trstring & s )
 {
@@ -136,10 +139,6 @@ struct TrError : std::runtime_error
 #undef CreateDirectory
 #endif
 
-#ifdef G_MAIN_GUI_DIR_H
-#error do not use dir.h from installer.cpp
-#endif
-
 struct ActionInterface
 {
 	virtual void run() = 0 ;
@@ -248,15 +247,21 @@ struct CreateSecrets : public ActionBase
 {
 	G::Path m_path ;
 	G::Path m_template ;
-	G::StringMap m_content ;
+	struct Item
+	{
+		std::string key ; // "client plain:b" or "server plain:b BOB==" or "server none"
+		std::string key2 ; // "client plain" or "server plain b+0B" or ""
+		std::string line ; // "client plain:b ALICE== PWD==" or "server plain:b BOB== PWD==" or "server none 127.0.0.1 trusted"
+	} ;
+	std::vector<Item> m_content ;
 	CreateSecrets( const std::string & config_dir , const std::string & filename , G::Path template_ , const G::MapFile & pvalues ) ;
 	void run() override ;
 	trstring text() const override ;
 	std::string subject() const override ;
 	static bool match( std::string , std::string ) ;
 	static bool yes( const std::string & ) ;
-	void addSecret( G::StringMap & , const G::MapFile & , const std::string & k ) const ;
-	void addSecret( G::StringMap & , const G::MapFile & , const std::string & side , const std::string & k1 , const std::string & k2 ) const ;
+	void addSecret( const G::MapFile & , const std::string & k ) ;
+	void addSecret( const G::MapFile & , const std::string & side , const std::string & k1 , const std::string & k2 ) ;
 	Q_DECLARE_TR_FUNCTIONS(CreateSecrets)
 } ;
 
@@ -502,7 +507,7 @@ void CreateDirectory::run()
 		{
 			G::File::mkdirs( m_path , 10 ) ;
 		}
-		Access::modify( m_path , m_tight_permissions ) ;
+		Gui::Access::modify( m_path , m_tight_permissions ) ;
 		if( !directory.writeable() )
 			throw TrError( tr("directory exists but is not writable") ) ;
 	}
@@ -554,7 +559,7 @@ void CreatePointerFile::run()
 	if( !isWindows() )
 	{
 		G::File::chmodx( m_pointer_file ) ;
-		G::File::chmodx( m_gui_exe ) ; // hopefully redundant
+		G::File::chmodx( m_gui_exe , std::nothrow ) ; // hopefully redundant
 	}
 }
 
@@ -704,18 +709,18 @@ CreateSecrets::CreateSecrets( const std::string & config_dir , const std::string
 {
 	if( yes(p.value("do-pop")) )
 	{
-		addSecret( m_content , p , "server" , "pop-auth-mechanism" , "pop-account-1" ) ;
-		addSecret( m_content , p , "server" , "pop-auth-mechanism" , "pop-account-2" ) ;
-		addSecret( m_content , p , "server" , "pop-auth-mechanism" , "pop-account-3" ) ;
+		addSecret( p , "server" , "pop-auth-mechanism" , "pop-account-1" ) ;
+		addSecret( p , "server" , "pop-auth-mechanism" , "pop-account-2" ) ;
+		addSecret( p , "server" , "pop-auth-mechanism" , "pop-account-3" ) ;
 	}
 	if( yes(p.value("do-smtp")) && yes(p.value("smtp-server-auth")) )
 	{
-		addSecret( m_content , p , "server" , "smtp-server-auth-mechanism" , "smtp-server-account" ) ;
-		addSecret( m_content , p , "smtp-server-trust" ) ;
+		addSecret( p , "server" , "smtp-server-auth-mechanism" , "smtp-server-account" ) ;
+		addSecret( p , "smtp-server-trust" ) ;
 	}
 	if( yes(p.value("do-smtp")) && yes(p.value("smtp-client-auth")) )
 	{
-		addSecret( m_content , p , "client" , "smtp-client-auth-mechanism" , "smtp-client-account" ) ;
+		addSecret( p , "client" , "smtp-client-auth-mechanism" , "smtp-client-account" ) ;
 	}
 }
 
@@ -724,32 +729,42 @@ bool CreateSecrets::yes( const std::string & s )
 	return G::Str::isPositive( s ) ;
 }
 
-void CreateSecrets::addSecret( G::StringMap & content , const G::MapFile & p , const std::string & k ) const
+void CreateSecrets::addSecret( const G::MapFile & p , const std::string & k )
 {
 	if( !p.value(k).empty() )
 	{
-		std::string head = "server none " + p.value(k) ;
-		std::string tail = "trusted" ;
-		content[head] = head + " " + tail ;
+		std::string address_range = p.value( k ) ;
+		m_content.push_back( {
+			"" ,
+			"" ,
+			"server none " + address_range + " trusted" } ) ;
 	}
 }
 
-void CreateSecrets::addSecret( G::StringMap & content , const G::MapFile & p ,
-	const std::string & side , const std::string & k1 , const std::string & k2 ) const
+void CreateSecrets::addSecret( const G::MapFile & p ,
+	const std::string & side , const std::string & k1_in , const std::string & k2 )
 {
 	if( !p.value(k2+"-name").empty() )
 	{
+		// see pages.cpp -- mechanism is "plain", name and secret are base64
+		//std::string mechanism = p.value( k1 ) ; // plain
+		std::string name_base64 = p.value( k2+"-name" ) ;
+		std::string secret_base64 = p.value( k2+"-password" ) ;
+		std::string name_xtext = G::Xtext::encode( G::Base64::decode(name_base64) ) ;
+		std::string secret_xtext = G::Xtext::encode( G::Base64::decode(secret_base64) ) ;
 		if( side == "server" )
 		{
-			std::string head = side + " " + p.value(k1) + " " + p.value(k2+"-name") ; // eg. "server plain joe"
-			std::string tail = p.value(k2+"-password") ; // eg. "secret"
-			content[head] = head + " " + tail ;
+			m_content.push_back( {
+				"server plain:b " + name_base64 ,
+				"server plain " + name_xtext ,
+				"server plain:b " + name_base64 + " " + secret_base64 } ) ;
 		}
 		else
 		{
-			std::string head = side + " " + p.value(k1) ; // eg. "client plain"
-			std::string tail = p.value(k2+"-name") + " " + p.value(k2+"-password") ; // eg. "joe secret"
-			content[head] = head + " " + tail ;
+			m_content.push_back( {
+				"client plain:b" ,
+				"client plain" ,
+				"client plain:b " + name_base64 + " " + secret_base64 } ) ;
 		}
 	}
 }
@@ -767,6 +782,7 @@ std::string CreateSecrets::subject() const
 bool CreateSecrets::match( std::string p1 , std::string p2 )
 {
 	// true if p1 starts with p2
+	if( p2.empty() ) return false ;
 	G::Str::replaceAll( p1 , "\t" , " " ) ;
 	while( G::Str::replaceAll( p1 , "  " , " " ) ) ;
 	G::Str::trimLeft( p1 , G::Str::ws() ) ;
@@ -811,7 +827,7 @@ void CreateSecrets::run()
 		if( line_list.empty() )
 		{
 			line_list.push_back( "#" ) ;
-			line_list.push_back( std::string() + "# " + m_path.basename() ) ;
+			line_list.push_back( "# " + m_path.basename() ) ;
 			line_list.push_back( "#" ) ;
 			line_list.push_back( "# client plain <name(xtext)> <password(xtext)>" ) ;
 			line_list.push_back( "# client plain:b <name(base64)> <password(base64)>" ) ;
@@ -830,16 +846,16 @@ void CreateSecrets::run()
 		bool replaced = false ;
 		for( auto & line : line_list )
 		{
-			if( match( line , map_item.first ) )
+			if( match(line,map_item.key) || match(line,map_item.key2) )
 			{
-				line = map_item.second ;
+				line = map_item.line ;
 				replaced = true ;
 				break ;
 			}
 		}
 		if( !replaced )
 		{
-			line_list.push_back( map_item.second ) ;
+			line_list.push_back( map_item.line ) ;
 		}
 	}
 
@@ -905,7 +921,7 @@ UpdateLink::UpdateLink( LinkType link_type , bool active , G::Path link_dir , G:
 		m_args(args) ,
 		m_icon(icon)
 {
-	std::string link_filename = G::Link::filename( "E-MailRelay" ) ;
+	std::string link_filename = Gui::Link::filename( "E-MailRelay" ) ;
 	m_link_path = G::Path( m_link_dir , link_filename ) ;
 }
 
@@ -928,8 +944,8 @@ void UpdateLink::run()
 	GDEF_UNUSED GComInit com_init ;
 	if( m_active )
 	{
-		G::Link link( m_target , "E-MailRelay" , "Starts the E-MailRelay server in the background" ,
-			m_working_dir , m_args , m_icon , G::Link::Show::Hide ,
+		Gui::Link link( m_target , "E-MailRelay" , "Starts the E-MailRelay server in the background" ,
+			m_working_dir , m_args , m_icon , Gui::Link::Show::Hide ,
 			"E-MailRelay" , "Generated by the E-MailRelay configuration GUI" ) ;
 
 		//G::Process::Umask umask( G::Process::Umask::Mode::Tightest ) ;
@@ -938,7 +954,7 @@ void UpdateLink::run()
 	}
 	else
 	{
-		m_ok = G::Link::remove( m_link_path ) ? tr("removed") : tr("nothing to do") ;
+		m_ok = Gui::Link::remove( m_link_path ) ? tr("removed") : tr("nothing to do") ;
 	}
 }
 
@@ -961,7 +977,7 @@ void InstallService::run()
 {
 	if( !m_active )
 	{
-		m_ok = tr("nothing to do") ; // see Boot::installable()
+		m_ok = tr("not possible") ; // see Gui::Boot::installable()
 	}
 	else if( m_bat.empty() || m_service_wrapper.empty() )
 	{
@@ -969,11 +985,11 @@ void InstallService::run()
 	}
 	else if( m_start_on_boot )
 	{
-		Boot::install( G::Path() , "emailrelay" , m_bat , m_service_wrapper ) ;
+		Gui::Boot::install( G::Path() , "emailrelay" , m_bat , m_service_wrapper ) ;
 	}
 	else
 	{
-		bool ok = Boot::uninstall( G::Path() , "emailrelay" , m_bat , m_service_wrapper ) ;
+		bool ok = Gui::Boot::uninstall( G::Path() , "emailrelay" , m_bat , m_service_wrapper ) ;
 		m_ok = ok ? tr("uninstalled") : tr("nothing to do") ;
 	}
 }
@@ -1019,7 +1035,7 @@ void UpdateBootLink::run()
 {
 	if( !m_active )
 	{
-		m_ok = tr("nothing to do") ; // see Boot::installable()
+		m_ok = tr("not possible") ; // see Gui::Boot::installable()
 	}
 	else if( m_dir_boot.empty() || m_startstop_src.empty() || m_exe.empty() )
 	{
@@ -1027,11 +1043,11 @@ void UpdateBootLink::run()
 	}
 	else if( m_start_on_boot )
 	{
-		Boot::install( m_dir_boot , m_name , m_startstop_src , m_exe ) ;
+		Gui::Boot::install( m_dir_boot , m_name , m_startstop_src , m_exe ) ;
 	}
 	else
 	{
-		bool removed = Boot::uninstall( m_dir_boot , m_name , m_startstop_src , m_exe ) ;
+		bool removed = Gui::Boot::uninstall( m_dir_boot , m_name , m_startstop_src , m_exe ) ;
 		m_ok = removed ? tr("removed") : tr("nothing to remove") ;
 	}
 }
@@ -1190,7 +1206,7 @@ G::Path GenerateKey::exe( bool is_windows )
 	return
 		this_exe.empty() ?
 			G::Path() :
-			G::Path(this_exe).dirname() + (is_windows?"emailrelay-test-keygen.exe":"emailrelay-test-keygen") ;
+			G::Path(this_exe).dirname() + (is_windows?"emailrelay_test_keygen.exe":"emailrelay_test_keygen") ;
 }
 
 void GenerateKey::run()
@@ -1250,17 +1266,17 @@ void Launcher::run()
 {
 	if( m_as_service )
 	{
-		Boot::launch( m_dir_boot , "emailrelay" ) ;
+		Gui::Boot::launch( m_dir_boot , "emailrelay" ) ;
 	}
 	else
 	{
-		// (keep it simple -- a console window will pop up to show 
-		// problems running the batch file and the emailrelay log 
+		// (keep it simple -- a console window will pop up to show
+		// problems running the batch file and the emailrelay log
 		// file is tail-ed by ProgressPage -- using G::NewProcess
-		// doesn't work well because stderr is not always inherited 
+		// doesn't work well because stderr is not always inherited
 		// and then closed cleanly)
 		std::string s = m_cmd.exe().str() ;
-		(void) system( s.c_str() ) ;
+		(void) system( s.c_str() ) ; // NOLINT
 	}
 }
 
@@ -1370,7 +1386,7 @@ InstallerImp::InstallerImp( bool installing , bool is_windows , bool is_mac , co
 	m_installer_config.add( "-exe" , isWindows() ? "%dir-install%/emailrelay.exe" :
 		( isMac() ? "%dir-install%/E-MailRelay.app/Contents/MacOS/emailrelay" : "%dir-install%/sbin/emailrelay" ) ) ;
 	m_installer_config.add( "-gui" , isWindows() ? "%dir-install%/emailrelay-gui.exe" : "%dir-install%/sbin/emailrelay-gui.real" ) ;
-	m_installer_config.add( "-icon" , isWindows()?"%dir-install%/emailrelay.exe":"%dir-install%/share/emailrelay/emailrelay-icon.png") ;
+	m_installer_config.add( "-icon" , isWindows()?"%dir-install%/emailrelay.exe":"%dir-install%/lib/emailrelay/emailrelay-icon.png");
 	m_installer_config.add( "-trdir" , isWindows()?"%dir-install%/translations":"%dir-install%/share/emailrelay") ;
 	m_installer_config.add( "-pointer" , isWindows() ? "%dir-install%/emailrelay-gui.cfg" : "%dir-install%/sbin/emailrelay-gui" ) ;
 	m_installer_config.add( "-startstop" , isWindows() ? "" : "%dir-install%/lib/emailrelay/emailrelay-startstop.sh" ) ;
@@ -1504,9 +1520,9 @@ void InstallerImp::addActions()
 	{
 		G::Path spool_dir( pvalue("dir-spool") ) ;
 		std::vector<std::string> names ;
-		names.push_back( pvalue("pop-account-1-name") ) ;
-		names.push_back( pvalue("pop-account-2-name") ) ;
-		names.push_back( pvalue("pop-account-3-name") ) ;
+		names.push_back( G::Xtext::encode( G::Base64::decode(pvalue("pop-account-1-name")) ) ) ;
+		names.push_back( G::Xtext::encode( G::Base64::decode(pvalue("pop-account-2-name")) ) ) ;
+		names.push_back( G::Xtext::encode( G::Base64::decode(pvalue("pop-account-3-name")) ) ) ;
 		for( const auto & name : names )
 		{
 			if( name.empty() ) continue ;
@@ -1555,6 +1571,8 @@ void InstallerImp::addActions()
 
 	// create secrets
 	//
+	// (this would be better using a separate file for the pop secrets)
+	//
 	G::Path authtemplate_src = m_installing ? ivalue( "-authtemplate" ) : std::string() ;
 	addAction( new CreateSecrets(pvalue("dir-config"),"emailrelay.auth",authtemplate_src,m_pages_output) ) ;
 
@@ -1607,29 +1625,35 @@ void InstallerImp::addActions()
 		}
 	}
 
-	// create a startup batch file
+	// update the configuration
 	//
 	if( isWindows() )
 	{
-		G::Path exe = ivalue("-exe") ;
-		G::Path bat = ivalue("-bat") ;
+		G::Path exe = ivalue( "-exe" ) ;
+		G::Path bat = ivalue( "-bat" ) ;
+		G::Path dir_install = pvalue( "dir-install" ) ;
+		G::Path working_dir = pvalue( "dir-config" ) ;
+		G::Path target = bat ;
+		G::Path icon = ivalue( "-icon" ) ;
 		G::StringArray args = ServerConfiguration::fromPages(m_pages_output).args() ;
 		addAction( new CreateBatchFile(bat,exe,args) ) ;
+		addAction( new UpdateLink(UpdateLink::LinkType::BatchFile,true,dir_install,working_dir,target,args,icon) ) ;
+	}
+	else
+	{
+		G::Path dir_config = pvalue( "dir-config" ) ;
+		G::Path conftemplate_src = m_installing ? ivalue( "-conftemplate" ) : std::string() ;
+		G::MapFile server_config = ServerConfiguration::fromPages(m_pages_output).map() ;
+		addAction( new CreateConfigFile(dir_config,"emailrelay.conf",conftemplate_src) ) ;
+		addAction( new EditConfigFile(dir_config,"emailrelay.conf",server_config,!m_installing) ) ;
 	}
 
-	// create startup links and startup config
+	// create startup links
 	//
+	if( yes(pvalue("start-page")) ) // if the startup page was presented at all -- see guimain.cpp
 	{
-		G::Path server_exe = ivalue("-exe") ;
-		G::Path working_dir = pvalue("dir-config") ;
-		G::Path dir_config = pvalue( "dir-config" ) ;
-		G::Path dir_install = pvalue( "dir-install" ) ;
-
-		bool do_desktop = yes(pvalue("start-link-desktop")) && !yes(pvalue("start-is-mac")) ;
-		bool do_menu = yes(pvalue("start-link-menu")) && !yes(pvalue("start-is-mac")) ;
-		bool do_login = yes(pvalue("start-at-login")) ;
-		bool do_boot = yes(pvalue("start-on-boot")) ;
-		bool do_boot_enabled = yes(pvalue("start-on-boot-enabled")) ; // see Boot::installable()
+		G::Path server_exe = ivalue( "-exe" ) ;
+		G::Path working_dir = pvalue( "dir-config" ) ;
 
 		G::Path dir_desktop = pvalue( "dir-desktop" ) ;
 		G::Path dir_menu = pvalue( "dir-menu" ) ;
@@ -1641,24 +1665,24 @@ void InstallerImp::addActions()
 		G::StringArray args = isWindows() ? G::StringArray() : ServerConfiguration::fromPages(m_pages_output).args() ;
 		G::Path icon = ivalue( "-icon" ) ;
 
-		addAction( new UpdateLink(UpdateLink::LinkType::Desktop,do_desktop,dir_desktop,working_dir,target,args,icon) ) ;
-		addAction( new UpdateLink(UpdateLink::LinkType::StartMenu,do_menu,dir_menu,working_dir,target,args,icon) ) ;
-		addAction( new UpdateLink(UpdateLink::LinkType::AutoStart,do_login,dir_login,working_dir,target,args,icon) ) ;
+		bool desktop_state = yes(pvalue("start-link-desktop")) && !yes(pvalue("start-is-mac")) ;
+		bool menu_state = yes(pvalue("start-link-menu")) && !yes(pvalue("start-is-mac")) ;
+		bool login_state = yes(pvalue("start-at-login")) ;
+		bool do_boot_update = yes(pvalue("start-on-boot-enabled")) ; // see Gui::Boot::installable()
+		bool boot_state = yes(pvalue("start-on-boot")) ;
+
+		addAction( new UpdateLink(UpdateLink::LinkType::Desktop,desktop_state,dir_desktop,working_dir,target,args,icon) ) ;
+		addAction( new UpdateLink(UpdateLink::LinkType::StartMenu,menu_state,dir_menu,working_dir,target,args,icon) ) ;
+		addAction( new UpdateLink(UpdateLink::LinkType::AutoStart,login_state,dir_login,working_dir,target,args,icon) ) ;
 
 		if( isWindows() )
 		{
-			addAction( new UpdateLink(UpdateLink::LinkType::BatchFile,true,dir_install,working_dir,target,args,icon) ) ;
 			G::Path service_wrapper = ivalue( "-servicewrapper" ) ;
-			addAction( new InstallService(do_boot_enabled,do_boot,bat,service_wrapper) ) ;
+			addAction( new InstallService(do_boot_update,boot_state,bat,service_wrapper) ) ;
 		}
 		else
 		{
-			// install the startstop script and its config file
-			G::Path conftemplate_src = m_installing ? ivalue( "-conftemplate" ) : std::string() ;
-			G::MapFile server_config = ServerConfiguration::fromPages(m_pages_output).map() ;
-			addAction( new UpdateBootLink(do_boot_enabled,do_boot,dir_boot,"emailrelay",ivalue("-startstop"),server_exe) ) ;
-			addAction( new CreateConfigFile(dir_config,"emailrelay.conf",conftemplate_src) ) ;
-			addAction( new EditConfigFile(dir_config,"emailrelay.conf",server_config,!m_installing) ) ;
+			addAction( new UpdateBootLink(do_boot_update,boot_state,dir_boot,"emailrelay",ivalue("-startstop"),server_exe) ) ;
 		}
 	}
 
