@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,9 +21,10 @@
 #include "gdef.h"
 #include "gqt.h"
 #include "pages.h"
-#include "legal.h"
-#include "dir.h"
-#include "boot.h"
+#include "guilegal.h"
+#include "guidialog.h"
+#include "guidir.h"
+#include "guiboot.h"
 #include "gmapfile.h"
 #include "gstr.h"
 #include "gfile.h"
@@ -34,31 +35,26 @@
 #include "gbase64.h"
 #include "gxtext.h"
 #include "gstringview.h"
+#include "genvironment.h"
 #include "glog.h"
 #include <stdexcept>
 #include <fstream>
 #include <cstring>
 
-namespace
-{
-	std::string encode( const std::string & s )
-	{
-		return G::Base64::encode( s ) ;
-	}
-}
+#include "moc_pages.cpp"
 
 // ==
 
-TitlePage::TitlePage( GDialog & dialog , const G::MapFile & , const std::string & name ,
+TitlePage::TitlePage( Gui::Dialog & dialog , const G::MapFile & , const std::string & name ,
 	const std::string & next_1 , const std::string & next_2 ) :
-		GPage(dialog,name,next_1,next_2)
+		Gui::Page(dialog,name,next_1,next_2)
 {
-	m_label = new QLabel( QString(Legal::text()) ) ;
+	m_label = new QLabel( QString(Gui::Legal::text()) ) ;
 	m_label->setAlignment( Qt::AlignHCenter ) ;
 
 	{
 		std::string credit = "<small><font color=\"#888\">" ;
-		credit.append( G::Str::join( "\n\n" , Legal::credits() ) ) ;
+		credit.append( G::Str::join( "\n\n" , Gui::Legal::credits() ) ) ;
 		credit.append( "</font></small>" ) ;
 		m_credit = new QLabel( QString(credit.c_str()) ) ;
 		m_credit->setAlignment( Qt::AlignHCenter ) ;
@@ -81,21 +77,21 @@ std::string TitlePage::nextPage()
 
 void TitlePage::dump( std::ostream & stream , bool for_install ) const
 {
-	GPage::dump( stream , for_install ) ;
+	Gui::Page::dump( stream , for_install ) ;
 }
 
 // ==
 
-LicensePage::LicensePage( GDialog & dialog , const G::MapFile & , const std::string & name ,
+LicensePage::LicensePage( Gui::Dialog & dialog , const G::MapFile & , const std::string & name ,
 	const std::string & next_1 , const std::string & next_2 , bool accepted ) :
-		GPage(dialog,name,next_1,next_2)
+		Gui::Page(dialog,name,next_1,next_2)
 {
 	m_text_edit = new QTextEdit ;
 	m_text_edit->setReadOnly( true ) ;
 	m_text_edit->setWordWrapMode( QTextOption::NoWrap ) ;
 	m_text_edit->setLineWrapMode( QTextEdit::NoWrap ) ;
 	m_text_edit->setFontFamily( "courier" ) ;
-	m_text_edit->setPlainText( QString(Legal::license()) ) ;
+	m_text_edit->setPlainText( QString(Gui::Legal::license()) ) ;
 
 	m_agree_checkbox = new QCheckBox( tr("I agree to the terms and conditions of the license") );
 	setFocusProxy( m_agree_checkbox ) ;
@@ -120,7 +116,7 @@ std::string LicensePage::nextPage()
 
 void LicensePage::dump( std::ostream & stream , bool for_install ) const
 {
-	GPage::dump( stream , for_install ) ;
+	Gui::Page::dump( stream , for_install ) ;
 }
 
 bool LicensePage::isComplete()
@@ -135,10 +131,10 @@ std::string LicensePage::helpUrl( const std::string & language ) const
 
 // ==
 
-DirectoryPage::DirectoryPage( GDialog & dialog , const G::MapFile & config , const std::string & name ,
+DirectoryPage::DirectoryPage( Gui::Dialog & dialog , const G::MapFile & config , const std::string & name ,
 	const std::string & next_1 , const std::string & next_2 ,
-	bool installing , bool is_mac ) :
-		GPage(dialog,name,next_1,next_2) ,
+	bool installing , bool /*is_windows*/ , bool is_mac ) :
+		Gui::Page(dialog,name,next_1,next_2) ,
 		m_installing(installing) ,
 		m_is_mac(is_mac) ,
 		m_other_dir_changed(false)
@@ -358,23 +354,23 @@ G::Path DirectoryPage::normalise( const G::Path & dir ) const
 	// leave them as relative to the bundle contents on mac
 
 	G::Path result = dir ;
-	if( dir.isRelative() && !Dir::home().empty() )
+	if( dir.isRelative() && !Gui::Dir::home().empty() )
 	{
 		if( dir.str() == "~" || dir.str() == "~/" || dir.str() == "$HOME" || dir.str() == "$HOME/" )
 		{
-			result = Dir::home() ;
+			result = Gui::Dir::home() ;
 		}
 		else if( dir.str().find("~/") == 0U )
 		{
-			result = G::Path( Dir::home() , dir.str().substr(2U) ) ;
+			result = G::Path( Gui::Dir::home() , dir.str().substr(2U) ) ;
 		}
 		else if( dir.str().find("$HOME/") == 0U )
 		{
-			result = G::Path( Dir::home() , dir.str().substr(6U) ) ;
+			result = G::Path( Gui::Dir::home() , dir.str().substr(6U) ) ;
 		}
 		else if( !m_is_mac )
 		{
-			result = G::Path::join( Dir::home() , dir ) ;
+			result = G::Path::join( Gui::Dir::home() , dir ) ;
 		}
 	}
 	return result ;
@@ -382,16 +378,16 @@ G::Path DirectoryPage::normalise( const G::Path & dir ) const
 
 void DirectoryPage::dump( std::ostream & stream , bool for_install ) const
 {
-	GPage::dump( stream , for_install ) ;
+	Gui::Page::dump( stream , for_install ) ;
 	dumpItem( stream , for_install , "dir-install" , installDir() ) ;
 	dumpItem( stream , for_install , "dir-spool" , spoolDir() ) ;
 	dumpItem( stream , for_install , "dir-config" , configDir() ) ;
 	dumpItem( stream , for_install , "dir-run" , runtimeDir() ) ;
 
-	dumpItem( stream , for_install , "dir-boot" , Dir::boot() ) ;
-	dumpItem( stream , for_install , "dir-desktop" , Dir::desktop() ) ;
-	dumpItem( stream , for_install , "dir-menu" , Dir::menu() ) ;
-	dumpItem( stream , for_install , "dir-login" , Dir::autostart() ) ;
+	dumpItem( stream , for_install , "dir-boot" , Gui::Dir::boot() ) ;
+	dumpItem( stream , for_install , "dir-desktop" , Gui::Dir::desktop() ) ;
+	dumpItem( stream , for_install , "dir-menu" , Gui::Dir::menu() ) ;
+	dumpItem( stream , for_install , "dir-login" , Gui::Dir::autostart() ) ;
 }
 
 bool DirectoryPage::isComplete()
@@ -424,9 +420,9 @@ G::Path DirectoryPage::configDir() const
 
 // ==
 
-DoWhatPage::DoWhatPage( GDialog & dialog , const G::MapFile & config , const std::string & name ,
+DoWhatPage::DoWhatPage( Gui::Dialog & dialog , const G::MapFile & config , const std::string & name ,
 	const std::string & next_1 , const std::string & next_2 ) :
-		GPage(dialog,name,next_1,next_2)
+		Gui::Page(dialog,name,next_1,next_2)
 {
 	m_pop_checkbox = new QCheckBox( tr("POP3 server") ) ;
 	m_smtp_checkbox = new QCheckBox( tr("SMTP server") ) ;
@@ -536,7 +532,7 @@ bool DoWhatPage::pop() const
 
 void DoWhatPage::dump( std::ostream & stream , bool for_install ) const
 {
-	GPage::dump( stream , for_install ) ;
+	Gui::Page::dump( stream , for_install ) ;
 	dumpItem( stream , for_install , "do-pop" , value(m_pop_checkbox) ) ;
 	dumpItem( stream , for_install , "do-smtp" , value(m_smtp_checkbox) ) ;
 	dumpItem( stream , for_install , "forward-immediate" , value(m_immediate_checkbox) ) ;
@@ -554,10 +550,10 @@ bool DoWhatPage::isComplete()
 
 // ==
 
-PopPage::PopPage( GDialog & dialog , const G::MapFile & config , const std::string & name ,
+PopPage::PopPage( Gui::Dialog & dialog , const G::MapFile & config , const std::string & name ,
 	const std::string & next_1 , const std::string & next_2 ,
 	bool have_accounts ) :
-		GPage(dialog,name,next_1,next_2) ,
+		Gui::Page(dialog,name,next_1,next_2) ,
 		m_have_accounts(have_accounts)
 {
 	//: internet address, port number
@@ -698,22 +694,22 @@ std::string PopPage::nextPage()
 
 void PopPage::dump( std::ostream & stream , bool for_install ) const
 {
-	GPage::dump( stream , for_install ) ;
+	Gui::Page::dump( stream , for_install ) ;
 	dumpItem( stream , for_install , "pop-port" , value(m_port_edit_box) ) ;
 	dumpItem( stream , for_install , "pop-simple" , value(m_one) ) ;
 	dumpItem( stream , for_install , "pop-shared" , value(m_shared) ) ;
 	dumpItem( stream , for_install , "pop-shared-no-delete" , value(m_no_delete_checkbox) ) ;
 	dumpItem( stream , for_install , "pop-by-name" , value(m_pop_by_name) ) ;
 
-	dumpItem( stream , for_install , "pop-auth-mechanism" , std::string("plain:b") ) ;
+	dumpItem( stream , for_install , "pop-auth-mechanism" , std::string("plain") ) ;
 	if( for_install )
 	{
-		dumpItem( stream , for_install , "pop-account-1-name" , encode(value_utf8(m_name_1)) ) ;
-		dumpItem( stream , for_install , "pop-account-1-password" , encode(value_utf8(m_pwd_1)) ) ;
-		dumpItem( stream , for_install , "pop-account-2-name" , encode(value_utf8(m_name_2)) ) ;
-		dumpItem( stream , for_install , "pop-account-2-password" , encode(value_utf8(m_pwd_2)) ) ;
-		dumpItem( stream , for_install , "pop-account-3-name" , encode(value_utf8(m_name_3)) ) ;
-		dumpItem( stream , for_install , "pop-account-3-password" , encode(value_utf8(m_pwd_3)) ) ;
+		dumpItem( stream , for_install , "pop-account-1-name" , G::Base64::encode(value_utf8(m_name_1)) ) ;
+		dumpItem( stream , for_install , "pop-account-1-password" , G::Base64::encode(value_utf8(m_pwd_1)) ) ;
+		dumpItem( stream , for_install , "pop-account-2-name" , G::Base64::encode(value_utf8(m_name_2)) ) ;
+		dumpItem( stream , for_install , "pop-account-2-password" , G::Base64::encode(value_utf8(m_pwd_2)) ) ;
+		dumpItem( stream , for_install , "pop-account-3-name" , G::Base64::encode(value_utf8(m_name_3)) ) ;
+		dumpItem( stream , for_install , "pop-account-3-password" , G::Base64::encode(value_utf8(m_pwd_3)) ) ;
 	}
 }
 
@@ -740,10 +736,10 @@ bool PopPage::withFilterCopy() const
 
 // ==
 
-SmtpServerPage::SmtpServerPage( GDialog & dialog , const G::MapFile & config , const std::string & name ,
+SmtpServerPage::SmtpServerPage( Gui::Dialog & dialog , const G::MapFile & config , const std::string & name ,
 	const std::string & next_1 , const std::string & next_2 ,
 	bool have_account , bool can_generate ) :
-		GPage(dialog,name,next_1,next_2) ,
+		Gui::Page(dialog,name,next_1,next_2) ,
 		m_have_account(have_account) ,
 		m_can_generate(can_generate)
 {
@@ -921,14 +917,14 @@ std::string SmtpServerPage::nextPage()
 
 void SmtpServerPage::dump( std::ostream & stream , bool for_install ) const
 {
-	GPage::dump( stream , for_install ) ;
+	Gui::Page::dump( stream , for_install ) ;
 	dumpItem( stream , for_install , "smtp-server-port" , value(m_port_edit_box) ) ;
 	dumpItem( stream , for_install , "smtp-server-auth" , value(m_auth_checkbox) ) ;
-	dumpItem( stream , for_install , "smtp-server-auth-mechanism" , std::string("plain:b") ) ;
+	dumpItem( stream , for_install , "smtp-server-auth-mechanism" , std::string("plain") ) ;
 	if( for_install )
 	{
-		dumpItem( stream , for_install , "smtp-server-account-name" , encode(value_utf8(m_account_name)) ) ;
-		dumpItem( stream , for_install , "smtp-server-account-password" , encode(value_utf8(m_account_pwd)) ) ;
+		dumpItem( stream , for_install , "smtp-server-account-name" , G::Base64::encode(value_utf8(m_account_name)) ) ;
+		dumpItem( stream , for_install , "smtp-server-account-password" , G::Base64::encode(value_utf8(m_account_pwd)) ) ;
 	}
 	dumpItem( stream , for_install , "smtp-server-trust" , value(m_trust_address) ) ;
 	dumpItem( stream , for_install , "smtp-server-tls" , value(m_tls_checkbox->isChecked() && m_tls_starttls->isChecked()) ) ;
@@ -967,10 +963,10 @@ bool SmtpServerPage::isComplete()
 
 // ==
 
-FilterPage::FilterPage( GDialog & dialog , const G::MapFile & config , const std::string & name ,
+FilterPage::FilterPage( Gui::Dialog & dialog , const G::MapFile & config , const std::string & name ,
 	const std::string & next_1 , const std::string & next_2 ,
 	bool installing , bool is_windows ) :
-		GPage(dialog,name,next_1,next_2) ,
+		Gui::Page(dialog,name,next_1,next_2) ,
 		m_installing(installing) ,
 		m_is_windows(is_windows) ,
 		m_dot_exe(is_windows?".exe":"") ,
@@ -1204,16 +1200,16 @@ void FilterPage::onToggle()
 
 void FilterPage::dump( std::ostream & stream , bool for_install ) const
 {
-	GPage::dump( stream , for_install ) ;
+	Gui::Page::dump( stream , for_install ) ;
 	dumpItem( stream , for_install , "filter-server" , value(m_server_filter_edit_box) ) ;
 	dumpItem( stream , for_install , "filter-client" , value(m_client_filter_edit_box) ) ;
 }
 
 // ==
 
-SmtpClientPage::SmtpClientPage( GDialog & dialog , const G::MapFile & config , const std::string & name ,
+SmtpClientPage::SmtpClientPage( Gui::Dialog & dialog , const G::MapFile & config , const std::string & name ,
 	const std::string & next_1 , const std::string & next_2 , bool have_account ) :
-		GPage(dialog,name,next_1,next_2) ,
+		Gui::Page(dialog,name,next_1,next_2) ,
 		m_have_account(have_account)
 {
 	//: internet address, hostname of remote smtp server
@@ -1351,17 +1347,17 @@ std::string SmtpClientPage::nextPage()
 
 void SmtpClientPage::dump( std::ostream & stream , bool for_install ) const
 {
-	GPage::dump( stream , for_install ) ;
+	Gui::Page::dump( stream , for_install ) ;
 	dumpItem( stream , for_install , "smtp-client-host" , value(m_server_edit_box) ) ;
 	dumpItem( stream , for_install , "smtp-client-port" , value(m_port_edit_box) ) ;
 	dumpItem( stream , for_install , "smtp-client-tls" , value(m_tls_checkbox->isChecked()&&!m_tls_tunnel->isChecked()) ) ;
 	dumpItem( stream , for_install , "smtp-client-tls-connection" , value(m_tls_checkbox->isChecked()&&m_tls_tunnel->isChecked()) ) ;
 	dumpItem( stream , for_install , "smtp-client-auth" , value(m_auth_checkbox) ) ;
-	dumpItem( stream , for_install , "smtp-client-auth-mechanism" , std::string("plain:b") ) ;
+	dumpItem( stream , for_install , "smtp-client-auth-mechanism" , std::string("plain") ) ;
 	if( for_install )
 	{
-		dumpItem( stream , for_install , "smtp-client-account-name" , encode(value_utf8(m_account_name)) ) ;
-		dumpItem( stream , for_install , "smtp-client-account-password" , encode(value_utf8(m_account_pwd)) ) ;
+		dumpItem( stream , for_install , "smtp-client-account-name" , G::Base64::encode(value_utf8(m_account_name)) ) ;
+		dumpItem( stream , for_install , "smtp-client-account-password" , G::Base64::encode(value_utf8(m_account_pwd)) ) ;
 	}
 }
 
@@ -1378,9 +1374,9 @@ bool SmtpClientPage::isComplete()
 
 // ==
 
-LoggingPage::LoggingPage( GDialog & dialog , const G::MapFile & config , const std::string & name ,
+LoggingPage::LoggingPage( Gui::Dialog & dialog , const G::MapFile & config , const std::string & name ,
 	const std::string & next_1 , const std::string & next_2 ) :
-		GPage(dialog,name,next_1,next_2)
+		Gui::Page(dialog,name,next_1,next_2)
 {
 	m_config_log_file = config.value( "log-file" ) ;
 
@@ -1523,7 +1519,7 @@ void LoggingPage::onToggle()
 
 void LoggingPage::dump( std::ostream & stream , bool for_install ) const
 {
-	GPage::dump( stream , for_install ) ;
+	Gui::Page::dump( stream , for_install ) ;
 	dumpItem( stream , for_install , "logging-verbose" , value(m_log_level_verbose_checkbox) ) ;
 	dumpItem( stream , for_install , "logging-debug" , value(m_log_level_debug_checkbox) ) ;
 	dumpItem( stream , for_install , "logging-syslog" , value(m_log_output_syslog_checkbox) ) ;
@@ -1534,9 +1530,10 @@ void LoggingPage::dump( std::ostream & stream , bool for_install ) const
 
 // ==
 
-ListeningPage::ListeningPage( GDialog & dialog , const G::MapFile & config , const std::string & name ,
-	const std::string & next_1 , const std::string & next_2 ) :
-		GPage(dialog,name,next_1,next_2)
+ListeningPage::ListeningPage( Gui::Dialog & dialog , const G::MapFile & config , const std::string & name ,
+	const std::string & next_1 , const std::string & next_2 , bool next_is_next2 ) :
+		Gui::Page(dialog,name,next_1,next_2) ,
+		m_next_is_next2(next_is_next2)
 {
 	//: server listening-address options...
 	m_all_checkbox = new QRadioButton( tr("Any address") ) ;
@@ -1637,7 +1634,7 @@ ListeningPage::ListeningPage( GDialog & dialog , const G::MapFile & config , con
 
 std::string ListeningPage::nextPage()
 {
-	return next1() ;
+	return m_next_is_next2 ? next2() : next1() ;
 }
 
 void ListeningPage::onTextChanged()
@@ -1680,16 +1677,16 @@ std::string ListeningPage::normalise( const std::string & s )
 
 void ListeningPage::dump( std::ostream & stream , bool for_install ) const
 {
-	GPage::dump( stream , for_install ) ;
+	Gui::Page::dump( stream , for_install ) ;
 	dumpItem( stream , for_install , "listening-interface" , normalise(value(m_listening_interface)) ) ;
 	dumpItem( stream , for_install , "listening-remote" , value(m_remote_checkbox) ) ;
 }
 
 // ==
 
-StartupPage::StartupPage( GDialog & dialog , const G::MapFile & config , const std::string & name ,
+StartupPage::StartupPage( Gui::Dialog & dialog , const G::MapFile & config , const std::string & name ,
 	const std::string & next_1 , const std::string & next_2 , bool is_mac ) :
-		GPage(dialog,name,next_1,next_2) ,
+		Gui::Page(dialog,name,next_1,next_2) ,
 		m_is_mac(is_mac)
 {
 	m_on_boot_checkbox = new QCheckBox( tr("At system startup") ) ;
@@ -1710,10 +1707,10 @@ StartupPage::StartupPage( GDialog & dialog , const G::MapFile & config , const s
 		m_add_menu_item_checkbox->setEnabled( false ) ;
 		m_add_desktop_item_checkbox->setEnabled( false ) ;
 	}
-	m_at_login_checkbox->setEnabled( !Dir::autostart().str().empty() ) ;
+	m_at_login_checkbox->setEnabled( !Gui::Dir::autostart().str().empty() ) ;
 	m_on_boot_checkbox->setEnabled( config.booleanValue("=dir-boot-enabled",false) ) ;
 
-	m_at_login_checkbox->setChecked( !Dir::autostart().str().empty() && config.booleanValue("start-at-login",false) ) ;
+	m_at_login_checkbox->setChecked( !Gui::Dir::autostart().str().empty() && config.booleanValue("start-at-login",false) ) ;
 	m_add_menu_item_checkbox->setChecked( !m_is_mac && config.booleanValue("start-link-menu",true) ) ;
 	m_add_desktop_item_checkbox->setChecked( !m_is_mac && config.booleanValue("start-link-desktop",false) ) ;
 	m_on_boot_checkbox->setChecked( config.booleanValue("start-on-boot",false) ) ;
@@ -1748,7 +1745,8 @@ std::string StartupPage::nextPage()
 
 void StartupPage::dump( std::ostream & stream , bool for_install ) const
 {
-	GPage::dump( stream , for_install ) ;
+	Gui::Page::dump( stream , for_install ) ;
+	dumpItem( stream , for_install , "start-page" , std::string("y") ) ; // since not necessarily used at all -- see guimain.cpp
 	dumpItem( stream , for_install , "start-on-boot-enabled" , std::string(m_on_boot_checkbox->isEnabled()?"y":"n") ) ;
 	dumpItem( stream , for_install , "start-on-boot" , value(m_on_boot_checkbox) ) ;
 	dumpItem( stream , for_install , "start-at-login" , value(m_at_login_checkbox) ) ;
@@ -1762,9 +1760,9 @@ void StartupPage::dump( std::ostream & stream , bool for_install ) const
 
 // ==
 
-ReadyPage::ReadyPage( GDialog & dialog , const G::MapFile & , const std::string & name , const std::string & next_1 ,
+ReadyPage::ReadyPage( Gui::Dialog & dialog , const G::MapFile & , const std::string & name , const std::string & next_1 ,
 	const std::string & next_2 , bool installing ) :
-		GPage(dialog,name,next_1,next_2) ,
+		Gui::Page(dialog,name,next_1,next_2) ,
 		m_installing(installing)
 {
 	m_label = new QLabel( text() ) ;
@@ -1802,7 +1800,7 @@ bool ReadyPage::isReadyToFinishPage() const
 
 void ReadyPage::dump( std::ostream & s , bool for_install ) const
 {
-	GPage::dump( s , for_install ) ;
+	Gui::Page::dump( s , for_install ) ;
 }
 
 // ==
@@ -1833,10 +1831,10 @@ void LogWatchThread::run()
 	}
 }
 
-ProgressPage::ProgressPage( GDialog & dialog , const G::MapFile & , const std::string & name ,
+ProgressPage::ProgressPage( Gui::Dialog & dialog , const G::MapFile & , const std::string & name ,
 	const std::string & next_1 , const std::string & next_2 ,
-	Installer & installer ) :
-		GPage(dialog,name,next_1,next_2) ,
+	Installer & installer , bool installing ) :
+		Gui::Page(dialog,name,next_1,next_2) ,
 		m_text_pos(0) ,
 		m_installer(installer) ,
 		m_state(0)
@@ -1848,8 +1846,8 @@ ProgressPage::ProgressPage( GDialog & dialog , const G::MapFile & , const std::s
 	//m_text_edit->setFontFamily( "courier" ) ;
 
 	auto * layout = new QVBoxLayout ;
-	//: page title of installation progress page
-	layout->addWidget( newTitle(tr("Installing")) ) ;
+	//: page title of installation-or-reconfiguration progress page
+	layout->addWidget( newTitle(installing?tr("Installing"):tr("Configuring")) ) ;
 	layout->addWidget(m_text_edit) ;
 	setLayout( layout ) ;
 }
@@ -1937,7 +1935,7 @@ void ProgressPage::onInstallTimeout()
 			if( m_installer.failed() )
 			{
 				if( m_state == 2 )
-					addLine( GQt::qstr(m_installer.failedText(),GQt::Utf8()) ) ;
+					addLine( GQt::qstr(m_installer.failedText(),GQt::Utf8) ) ;
 				else
 					m_installer.back() ;
 				m_state += 1 ;
@@ -1945,13 +1943,13 @@ void ProgressPage::onInstallTimeout()
 			else
 			{
 				if( m_state == 2 )
-					addLine( GQt::qstr(m_installer.finishedText(),GQt::Utf8()) ) ;
+					addLine( GQt::qstr(m_installer.finishedText(),GQt::Utf8) ) ;
 				m_state += 2 ;
 				if( m_logwatch_thread )
 					m_logwatch_thread->start() ;
 			}
 		}
-		emit pageUpdateSignal() ; // NOLINT
+		emit pageUpdateSignal() ;
 	}
 	catch( std::exception & e )
 	{
@@ -1978,10 +1976,10 @@ void ProgressPage::addOutput( const Installer::Output & output )
 
 QString ProgressPage::format( const Installer::Output & output )
 {
-	QString action_tr = GQt::qstr( output.action_utf8 , GQt::Utf8() ) ;
+	QString action_tr = GQt::qstr( output.action_utf8 , GQt::Utf8 ) ;
 	QString subject_native = GQt::qstr( output.subject ) ;
-	QString result_tr = GQt::qstr( output.result_utf8 , GQt::Utf8() ) ;
-	QString error_tr = GQt::qstr( output.error_utf8 , GQt::Utf8() ) ;
+	QString result_tr = GQt::qstr( output.result_utf8 , GQt::Utf8 ) ;
+	QString error_tr = GQt::qstr( output.error_utf8 , GQt::Utf8 ) ;
 	QString error_native = GQt::qstr( output.error ) ;
 
 	// use the translation mechanism to format the line because
@@ -2060,7 +2058,7 @@ std::string ProgressPage::nextPage()
 
 void ProgressPage::dump( std::ostream & s , bool for_install ) const
 {
-	GPage::dump( s , for_install ) ;
+	Gui::Page::dump( s , for_install ) ;
 }
 
 bool ProgressPage::isFinishPage() const
