@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #define G_MAIN_GUI_SERVER_CONFIGURATION_H
 
 #include "gdef.h"
-#include "gstrings.h"
+#include "gstringarray.h"
 #include "gmapfile.h"
 #include "gpath.h"
 
