@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 // 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
 #include "gdef.h"
 #include "goptions.h"
 #include <string>
+#include <vector>
+#include <utility>
 
 namespace Main
 {
@@ -40,6 +42,12 @@ public:
 	static G::Options spec( bool is_windows ) ;
 		///< Returns an o/s-specific G::OptionParser specification.
 
+	using Tag = std::pair<unsigned,std::string> ;
+
+	static std::vector<Tag> tags() ;
+		///< Returns an ordered list of tags to be matched
+		///< against each option's 'main_tag'.
+
 public:
 	Options() = delete ;
 } ;
