@@ -3,16 +3,13 @@
 #define IDD_DIALOG1                     101
 #define IDR_MENU1                       102
 #define IDD_DIALOG2                     103
-#define IDI_ICON2                       105
-#define IDI_ICON3                       106
-#define IDI_ICON4                       107
-#define IDI_ICON5                       110
+#define IDD_PROPPAGE_1                  106
 #define IDC_EDIT1                       1000
+#define IDC_LIST1                       1000
 #define IDS_EVENTLOG_INFORMATION        1001
 #define IDS_EVENTLOG_WARNING            1002
 #define IDS_EVENTLOG_ERROR              1003
 #define IDM_OPEN                        40001
 #define IDM_QUIT                        40002
 #define IDM_CLOSE                       40005
-#define IDC_STATIC                      -1
 
