@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright (C) 2001-2013 Graeme Walker <graeme_walker@users.sourceforge.net>
+# Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,10 +20,20 @@
 #
 # A network client to drive the admin interface.
 #
+# Synopsis:
+#
+#	my $ac = new AdminClient(10026,"localhost") ;
+#	$ac->open() or die ; # first
+#	$rsp = $ac->doHelp() ;
+#	$ac->doTerminate() ;
+#	$ac->doFlush() ;
+#	$line = $ac->getline() ; # last
+#
 
 use strict ;
 use FileHandle ;
-use Net::Telnet ;
+use NetClient ;
+use System ;
 
 package AdminClient ;
 
@@ -31,49 +41,41 @@ sub new
 {
 	my ( $classname , $port , $server ) = @_ ;
 
-	$server = defined($server) ? $server : "localhost" ;
-	$port = defined($port) ? $port : 10026 ;
-
-	my $t = new Net::Telnet( Timeout=>3 , Prompt=>"/E-MailRelay> /" ) ;
-	$t->max_buffer_length(1000000*10) ;
+	$port ||= 10026 ;
+	$server ||= $System::localhost ;
 
 	my %me = (
 		m_port => $port ,
 		m_server => $server ,
-		m_t => $t ,
+		m_prompt => qr/E-MailRelay> / ,
+		m_timeout => 3 ,
+		m_s => undef ,
 	) ;
 	return bless \%me , $classname ;
 }
 
+sub port { return shift->{m_port} }
+sub server { return shift->{m_server} }
+sub doHelp { return NetClient::cmd( $_[0] , "help" ) }
+sub doTerminate { NetClient::send( $_[0] , "terminate\r\n" ) }
+sub doFlush { NetClient::send( $_[0] , "flush\r\n") }
+
 sub open
 {
 	my ( $this ) = @_ ;
-	return $this->{'m_t'}->open( Host=>$this->server() , Port=>$this->port() ) ;
+	$this->{m_s} = NetClient::newSocket( $this ) ;
+	return !! $this->{m_s} ;
 }
 
-sub port { return shift->{m_port} }
-sub server { return shift->{m_server} }
-sub doHelp { shift->{'m_t'}->cmd("help") }
-sub doTerminate { shift->{'m_t'}->print("terminate") }
-sub doFlush { shift->{'m_t'}->print("flush") }
-sub getline { return shift->{'m_t'}->getline() }
-
-sub doList 
-{ 
-	my ( $this ) = @_ ;
-	$this->{'m_t'}->buffer_empty() ; # just in case
-	my @output = $this->{'m_t'}->cmd("list") ;
-	my @result = () ;
-	for my $s ( @output )
-	{
-		chomp $s ;
-		if( $s ne "<none>" )
-		{
-			push @result , $s ;
-		}
-	}
-	return @result ;
+sub getline
+{
+	my ( $this , $timeout ) = @_ ;
+	$timeout ||= $this->{m_timeout} ;
+	my $line = NetClient::read( $this , qr/\n/ , $timeout ) ;
+	$line = "" if !defined($line) ;
+	$line =~ s/\r//g ;
+	$line =~ s/\n$// ;
+	return $line ;
 }
 
 1 ;
-
