@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright (C) 2001-2013 Graeme Walker <graeme_walker@users.sourceforge.net>
+# Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
 #
 # Check.pm
 #
-# Various assertion functions that all resolve
-# down to Check::that(ok,text).
+# Various assertion functions that all reduce down
+# to Check::that(ok,text).
 #
 
 use strict ;
@@ -33,7 +33,7 @@ sub that
 	my ( $ok , @args ) = @_ ;
 	if( !$ok )
 	{
-		Carp::croak( "[ " . join(": ",@args) . " ]" ) ;
+		Carp::croak( "[ " . join(": ",map {my $s=$_=~s/\n/\\n/gr=~s/\r/\\r/gr;$s} grep{defined($_)} @args) . " ]" ) ;
 	}
 }
 
@@ -68,21 +68,21 @@ sub fileNotEmpty
 sub running
 {
 	my ( $pid , $more ) = @_ ;
-	my $n = defined($pid) ? System::processIsRunning($pid) : -1 ;
+	my $n = defined($pid) ? System::processIsRunning($pid) : 0 ;
 	Check::that( $n == 1 , "process not running" , $pid , $more ) ;
 }
 
 sub notRunning
 {
 	my ( $pid , $more ) = @_ ;
-	my $n = System::processIsRunning($pid) ;
+	my $n = defined($pid) ? System::processIsRunning($pid) : 0 ;
 	Check::that( $n == 0 , "process still running" , $pid , $more ) ;
 }
 
 sub numeric
 {
-	my ( $s ) = @_ ;
-	Check::that( defined($s) && $s =~ m/[[:digit:]]+/ , "invalid numeric string" , $s ) ;
+	my ( $s , $more ) = @_ ;
+	Check::that( defined($s) && $s =~ m/[[:digit:]]+/ , "invalid numeric string" , "[$s]" , $more ) ;
 }
 
 sub fileDeleted
@@ -96,7 +96,7 @@ sub fileMatchCount
 	my ( $expr , $count , $more ) = @_ ;
 	my @files = System::glob_( $expr ) ;
 	my $n = scalar(@files) ;
-	Check::that( $n == $count , "unexpected number of matching files ($n != $count)" , $more ) ;
+	Check::that( $n == $count , "unexpected number of matching files (got $n, expected $count) [$expr]" , $more ) ;
 }
 
 sub fileOwner
@@ -136,7 +136,7 @@ sub processEffectiveUser
 	my ( $pid , $name ) = @_ ;
 	my $actual = System::effectiveUser($pid) ;
 	my $expected = System::uid($name) ;
-	Check::that( $actual == $expected , "wrong real user: [$actual]!=[$expected]" ) ;
+	Check::that( $actual == $expected , "wrong effective user: [$actual]!=[$expected]" ) ;
 }
 
 sub processSavedUser
@@ -144,7 +144,7 @@ sub processSavedUser
 	my ( $pid , $name ) = @_ ;
 	my $actual = System::savedUser($pid) ;
 	my $expected = System::uid($name) ;
-	Check::that( $actual == $expected , "wrong real user: [$actual]!=[$expected]" ) ;
+	Check::that( $actual == $expected , "wrong saved user: [$actual]!=[$expected]" ) ;
 }
 
 sub processRealGroup
@@ -160,15 +160,15 @@ sub processEffectiveGroup
 	my ( $pid , $name ) = @_ ;
 	my $actual = System::effectiveGroup($pid) ;
 	my $expected = System::gid($name) ;
-	Check::that( $actual == $expected , "wrong real group: [$actual]!=[$expected]" ) ;
+	Check::that( $actual == $expected , "wrong effective group: [$actual]!=[$expected]" ) ;
 }
 
 sub _fileLineCount
 {
 	my ( $path , $string ) = @_ ;
-	my $f = new FileHandle( $path ) ;
+	my $fh = new FileHandle( $path ) ;
 	my $n = 0 ;
-	while( <$f> )
+	while(<$fh>)
 	{
 		my $line = $_ ;
 		chomp $line ;
@@ -193,34 +193,45 @@ sub fileLineCountLessThan
 
 sub allFilesContain
 {
-	my ( $glob , $string , $more ) = @_ ;
+	my ( $glob , $re , $more ) = @_ ;
 	my @files = System::glob_( $glob ) ;
 	for my $file ( @files )
 	{
-		fileContains( $file , $string , $more ) ;
+		fileContains( $file , $re , $more ) ;
+	}
+}
+
+sub noFileContains
+{
+	my ( $glob , $string_or_list , $more ) = @_ ;
+	my @files = System::glob_( $glob ) ;
+	for my $file ( @files )
+	{
+		fileDoesNotContain( $file , $string_or_list , $more ) ;
 	}
 }
 
 sub fileContains
 {
-	my ( $path , $string , $more ) = @_ ;
-	my $f = new FileHandle( $path ) ;
+	my ( $path , $re , $more , $count ) = @_ ;
+	my $fh = new FileHandle( $path ) ;
 	my $n = 0 ;
-	while( <$f> )
+	while(<$fh>)
 	{
 		my $line = $_ ;
 		chomp $line ;
-		if( !defined($string) || $line =~ m/$string/ ) { $n++ }
+		if( !defined($re) || $line =~ m/$re/ ) { $n++ }
 	}
-	Check::that( $n > 0 , "file does not contain expected string" , $path , "[$string]" , $more ) ;
+	my $ok = defined($count) ? ($n == $count) : ($n > 0) ;
+	Check::that( $ok , "file does not contain expected string" , $path , "[$re]" , $more ) ;
 }
 
 sub fileContainsEither
 {
 	my ( $path , $string1 , $string2 , $more ) = @_ ;
-	my $f = new FileHandle( $path ) ;
+	my $fh = new FileHandle( $path ) ;
 	my $n = 0 ;
-	while( <$f> )
+	while(<$fh>)
 	{
 		my $line = $_ ;
 		chomp $line ;
@@ -232,16 +243,21 @@ sub fileContainsEither
 
 sub fileDoesNotContain
 {
-	my ( $path , $string , $more ) = @_ ;
-	my $f = new FileHandle( $path ) ;
+	my ( $path , $string_or_list , $more ) = @_ ;
+	die if !defined($string_or_list) ;
+	my @strings = ref($string_or_list) ? @$string_or_list : ($string_or_list) ;
+	my $fh = new FileHandle( $path ) ;
 	my $n = 0 ;
-	while( <$f> )
+	while(<$fh>)
 	{
 		my $line = $_ ;
 		chomp $line ;
-		if( !defined($string) || $line =~ m/$string/ ) { $n++ }
+		for my $string ( @strings )
+		{
+			if( $line =~ m/$string/ ) { $n++ }
+		}
 	}
-	Check::that( $n <= 0 , "file contains unexpected string" , $path , "[$string]" , $more ) ;
+	Check::that( $n <= 0 , "file contains unexpected string" , $path , "[@strings]" , $more ) ;
 }
 
 sub match
@@ -252,4 +268,3 @@ sub match
 }
 
 1 ;
-
