@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright (C) 2001-2013 Graeme Walker <graeme_walker@users.sourceforge.net>
+# Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,9 @@
 #
 # Scanner.pm
 #
-# A wrapper for running the "emailrelay-test-scanner"
-# program.
+# A wrapper for running the "emailrelay_test_scanner" program.
+#
+# See also: Helper
 #
 
 use strict ;
@@ -29,8 +30,8 @@ package Scanner ;
 
 sub new
 {
-	my ( $classname , $port ) = @_ ;
-	return bless { h => new Helper( "scanner" , $port ) } , $classname ;
+	my ( $classname , $address ) = @_ ;
+	return bless { h => new Helper( "emailrelay_test_scanner" , $address ) } , $classname ; # "--port <address>"
 }
 
 sub port { return shift->{h}->port(@_) }
