@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright (C) 2001-2013 Graeme Walker <graeme_walker@users.sourceforge.net>
+# Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,14 +18,24 @@
 #
 # TestServer.pm
 #
-# A wrapper for running the "emailrelay-test-server" program.
+# A wrapper for running the "emailrelay_test_server" program.
+#
+# See also: emailrelay_test_server.cpp
+#
+# Synopsis:
+#
+#	use TestServer ;
+#	$TestServer::bin_dir = "." ;
+#	my $ts = new TestServer( 10025 ) ;
+#	$ts->run() ;
+#	$ts->kill() ;
+#	$ts->cleanup() ;
+#	kill 15 , @TestServer::pid_list ;
 #
 
 use strict ;
 use FileHandle ;
-use Scanner ;
 use System ;
-use Port ;
 
 package TestServer ;
 
@@ -40,9 +50,9 @@ sub new
 
 	my %me = (
 		m_port => $port ,
-		m_exe => System::exe( $bin_dir , "emailrelay-test-server" ) ,
-		m_logfile => "$bin_dir/.tmp.emailrelay-test-server.out" ,
-		m_pidfile => "$bin_dir/.emailrelay-test-server.pid" ,
+		m_exe => System::exe( $bin_dir , "emailrelay_test_server" ) ,
+		m_logfile => System::tempfile("test-server.out") ,
+		m_pidfile => System::tempfile("test-server.pid") ,
 		m_pid => undef ,
 	) ;
 	my $this = bless \%me , $classname ;
@@ -50,9 +60,9 @@ sub new
 	return $this ;
 }
 
-sub exe { return shift->{'m_exe'} }
-sub pid { return shift->{'m_pid'} }
-sub port { return shift->{'m_port'} }
+sub exe { return shift->{m_exe} }
+sub pid { return shift->{m_pid} }
+sub port { return shift->{m_port} }
 
 sub _check
 {
@@ -61,11 +71,7 @@ sub _check
 	{
 		die "invalid test server executable [".$this->exe()."]" ;
 	}
-	if( ! Port::isFree($this->port(),Port::list()) )
-	{
-		die "port " . $this->port() . " not free" ;
-	}
-	unlink( $this->{m_pidfile} ) if -f $this->{m_pidfile} ;
+	System::unlink( $this->{m_pidfile} ) if -f $this->{m_pidfile} ;
 	if( -f $this->{m_pidfile} )
 	{
 		die "cannot remove old pidfile [".$this->{m_pidfile}."]" ;
@@ -77,48 +83,32 @@ sub run
 	my ( $this , $sw , $wait_cs ) = @_ ;
 	$sw = "" if !defined($sw) ;
 	$wait_cs ||= 50 ;
-	my $log = $this->{'m_logfile'} ;
+	my $log = $this->{m_logfile} ;
 
-	my $cmd = System::weirdpath($this->exe()) . " --port " . $this->port() . " $sw" ;
+	my $cmd = System::sanepath($this->exe()) . " --port " . $this->port() . " --pid-file " . $this->{m_pidfile} . " $sw" ;
 	my $full_cmd = System::commandline( $cmd , { stdout => $log , stderr => $log , background => 1 } ) ;
+	System::log_( "running [$full_cmd]" ) ;
 	system( $full_cmd ) ;
 
-	# wait for the pid file to be written
-	for( my $i = 0 ; $i < $wait_cs ; $i++ )
-	{
-		select( undef , undef , undef , 0.01 ) ;
-		my $fh = new FileHandle( $this->{'m_pidfile'} ) ;
-		next if !$fh ;
-		my $line = <$fh> ;
-		chomp $line ;
-		if( $line > 0 )
-		{
-			my $pid = $line ;
-			$this->{'m_pid'} = $pid ;
-			push @pid_list , $pid ;
-			last ;
-		}
-	}
-
-	return System::processIsRunning( $this->{'m_pid'} ) ;
+	my $pid = System::waitForPid( $this->{m_pidfile} ) ;
+	$this->{m_pid} = $pid ;
+	push @pid_list , $pid if $pid ;
+	return $pid && System::processIsRunning($pid) ;
 }
 
 sub kill
 {
 	my ( $this ) = @_ ;
-	if( defined($this->pid()) )
-	{
-		System::kill_( $this->pid() ) ;
-		System::sleep_cs( 10 ) ;
-	}
+	Check::numeric( $this->pid() ) ;
+	System::kill_( $this->pid() ) ;
 }
 
 sub cleanup
 {
 	my ( $this ) = @_ ;
-	$this->kill() ;
-	unlink( $this->{'m_pidfile'} ) ;
-	unlink( $this->{'m_logfile'} ) ;
+	if( $this->pid() ) { $this->kill() }
+	System::unlink( $this->{m_pidfile} ) ;
+	System::unlink( $this->{m_logfile} ) ;
 }
 
 1 ;
