@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright (C) 2001-2013 Graeme Walker <graeme_walker@users.sourceforge.net>
+# Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,9 @@
 #
 # Verifier.pm
 #
-# A wrapper for running the "emailrelay-test-verifier"
-# program.
+# A wrapper for running the "emailrelay_test_verifier" program.
+#
+# See also: Helper
 #
 
 use strict ;
@@ -30,7 +31,7 @@ package Verifier ;
 sub new
 {
 	my ( $classname , $port ) = @_ ;
-	return bless { h => new Helper( "verifier" , $port ) } , $classname ;
+	return bless { h => new Helper( "emailrelay_test_verifier" , $port ) } , $classname ;
 }
 
 sub port { return shift->{h}->port(@_) }
